/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: GenerateDialog.cpp,v 2.5 2001/03/13 20:34:15 kebner Exp $

*******************************************************************************/


#include <qdir.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qfiledialog.h>
#include <qpushbt.h>
#include <qmsgbox.h>
#include <qlabel.h>

#include "GenerateDialog.h"

#define Inherited GenerateDialogData

GenerateDialog::GenerateDialog
(
    const QString&	target,
    const QString&	fileName,
    const QString&	dlgClassName,
    const QString&	classHeader,
    const QString&	classSource,
    const QString&	dataHeader,
    const QString&	dataSource,
	QWidget* parent,
	const char* name
)
	:
	Inherited( parent, name )
{
    setCaption( tr( "Generate Source" ) );

    dialogFileNameLabel->setText( tr( "Dialog file: %1" ).arg( fileName ) );
    dialogClassNameLabel->setText( tr( "Dialog class: %1" ).arg( dlgClassName ) );

    itsClassHeaderEdit->setText( classHeader );
    itsClassSourceEdit->setText( classSource );
    itsDataHeaderEdit->setText( dataHeader );
    itsDataSourceEdit->setText( dataSource );

    itsTargetDir->setText( targetDir = target );

    CheckGenerateSource();
    
    itsDataCheckbox->setChecked( TRUE );
    GetDataNames( TRUE );
}

GenerateDialog::~GenerateDialog()
{
}

void GenerateDialog::GetSourceNames(bool on)
{
    itsClassHeaderLabel->setEnabled( on );
    itsClassSourceLabel->setEnabled( on );
    itsClassHeaderEdit->setEnabled( on );
    itsClassSourceEdit->setEnabled( on );
}

void GenerateDialog::BrowseTargetDir()
{
  targetDir = QFileDialog::getExistingDirectory( itsTargetDir->text(),
                                                 this,
                                                 0,
                                                 tr( "New target directory" ),
                                                 true );
  itsTargetDir->setText( targetDir );

  CheckGenerateSource();
}

void GenerateDialog::GetDataNames(bool on)
{
    itsDataHeaderLabel->setEnabled( on );
    itsDataSourceLabel->setEnabled( on );
    itsDataHeaderEdit->setEnabled( on );
    itsDataSourceEdit->setEnabled( on );
}

void GenerateDialog::TargetDirChanged( const QString& newDir )
{
  targetDir = newDir;

  CheckGenerateSource();
}

/* Slot called by the OK-Button
 *
 * Check wether all LineEdits which are visible have a text
 * If so, end the dialog with accept, otherwise bring up a messagebox
 * which explains the horrible error which was made by the user...
 */
void GenerateDialog::EndDialog()
{
  QString sC, hC;
  QString sD, hD;

  int errorCode = 0;

  if( itsSourceCheckbox->isChecked() )
  {
    hC = itsClassHeaderEdit->text();
    sC = itsClassSourceEdit->text();
  }

  if( itsDataCheckbox->isChecked() )
  {
    hD = itsDataHeaderEdit->text();
    sD = itsDataSourceEdit->text();
  }

  if( itsSourceCheckbox->isChecked() )
  {
    if( sC == "" )
      errorCode |= 1;
    if( hC == "" )
      errorCode |= 2;
  }
  if( itsDataCheckbox->isChecked() )
  {
    if( sD == "" )
      errorCode |= 4;
    if( hD == "" )
      errorCode |= 8;
  }

  // Some of the following error conditions are, mh, absurd
  QString errorText;
  switch( errorCode )
  {
  case 0:
    accept();
    break;
  case 1:
    errorText = tr( "No file for the class header specified" );
    break;
  case 2:
    errorText = tr( "No file for the class source specified" );
    break;
  case 3:
    errorText = tr( "No files for the class specified" );
    break;
  case 4:
    errorText = tr( "No file for the data header specified" );
    break;
  case 5:
    errorText = tr( "No file for the headers specified" );
    break;
  case 6:
    errorText = tr( "No file for the class source and the data header specified" );
    break;
  case 7:
    errorText = tr( "No file for the class and the data header specified" );
    break;
  case 8:
    errorText = tr( "No file for the data source specified" );
    break;
  case 9:
    errorText = tr( "No file for the class header and data source specified" );
    break;
  case 10:
    errorText = tr( "No files for the sources specified" );
    break;
  case 11:
    errorText = tr( "No files for the class and the data source specified" );
    break;
  case 12:
    errorText = tr( "No files for the data specified" );
    break;
  case 13:
    errorText = tr( "No files for the class header and the data specified" );
    break;
  case 14:
    errorText = tr( "No files for the class source and the data specified" );
    break;
  case 15:
    errorText = tr( "No files specified" );
    break;
  }

  if( errorCode )
  {
      QMessageBox::message( tr( "Error..." ),
			    errorText,
			    tr( "Try again" ),
			    this );
  }
}

bool GenerateDialog::GenerateSource( QString &header, QString &source )
{
  bool ret = itsSourceCheckbox->isChecked();
  if( ret )
  {
    header = itsClassHeaderEdit->text();
    source = itsClassSourceEdit->text();
  }
  return ret; 
}

bool GenerateDialog::GenerateData( QString &header, QString &source )
{
  bool ret = itsDataCheckbox->isChecked();
  if( ret )
  {
    header = itsDataHeaderEdit->text();
    source = itsDataSourceEdit->text();
  }
  return ret; 
}


void GenerateDialog::CheckGenerateSource()
{
  QDir fileDir( targetDir );
    
  bool checked = 
    (itsClassHeaderEdit->text().isEmpty() || 
     !QFile::exists( QFileInfo( fileDir, itsClassHeaderEdit->text() ).filePath() )) &&
    (itsClassSourceEdit->text().isEmpty() || 
     !QFile::exists( QFileInfo( fileDir, itsClassSourceEdit->text() ).filePath() ));

  itsSourceCheckbox->setChecked( checked );
  GetSourceNames( checked );
}









