/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: Project.cpp,v 2.9 2000/07/23 20:36:43 jeffharris Exp $

*******************************************************************************/


#include <qfile.h>
#include <qlist.h>
#include <qmsgbox.h>
#include <qfileinf.h>

#include "Project.h"

#include "ProjectWnd.h"
#include "Dialog.h"
#include "pixmap/Pixmap.h"
#include "ProjectDialog.h"
#include "TMakeDialog.h"
#include "MakeEdit.h"

#include "DlgParser.h"
#include "xml/QtArchGenerator.h"
#include "xml/QtArchExpat.h"

#include "stdlib.h"

#ifndef _WIN32
#include "unistd.h"
#include "sys/types.h"
#include "sys/wait.h"
#endif

void SpawnTMake( QString &makeFileName, QString &profile )
{
#ifndef _WIN32
  int pid = fork();

  if( pid )
  {
    int childId = waitpid( pid, 0, 0 );
    if( childId != pid )
    {
      debug( "pid = %d, childId = %d", pid, childId );
      perror( "Wait failed" );
    }

  } else
  {
    execlp( "tmake", 
            "tmake", 
            "-o", (const char *) makeFileName,
            (const char *) profile, NULL );
    perror( "execl to tmake failed" );
  }
#endif
}

Project::Project( ProjectWnd &projectWnd, const QString &name )
  : DlgObject( projectWnd, name )
{
  /* Create three popupmenus for the List:
   * 1. Not in a project
   * 2. In a project, not hidden
   * 3. In a project, hidden
   */
  unboundPopup = new QPopupMenu;
  unboundPopup->insertItem( tr( "Rename Project" ), this, SLOT(Rename()) );
  unboundPopup->insertSeparator();
  unboundPopup->insertItem( tr( "Save Project" ), this, SLOT(Save()) );
  unboundPopup->insertItem( tr( "Save Project As..." ), this, SLOT(SaveAs()));
  unboundPopup->insertItem( tr( "Close Project" ), this, SLOT(Hide()) );
  unboundPopup->insertItem( tr( "Delete" ), this, SLOT(Delete()) );
  unboundPopup->insertSeparator();
  unboundPopup->insertItem( tr( "Add Object..." ), this, SLOT(AddObject()) );
  unboundPopup->insertItem( tr( "Generate Objects" ), this, SLOT(Generate()) );

  // Not yet implemented
  unboundHiddenPopup = NULL;
  boundPopup = NULL;
  boundHiddenPopup = NULL;

  // Will hold the generated files
  genSources.setAutoDelete( TRUE );
  genHeaders.setAutoDelete( TRUE );
  genMeta.setAutoDelete( TRUE );

  makefile.setAutoDelete( TRUE );

};

bool Project::Load()
{
  if( ! itsFilename.isEmpty() )
  {
    if( fileTypeXML )
    {
      QtArchExpat parser( itsFilename );
      connect( &parser,
               SIGNAL(NewType(const XMLValue&)),
               SLOT(RestoreXML(const XMLValue&)) );
      parser.Parse();
    } else
    {
      itsParser = new DlgParser( itsFilename );
      if( ! itsParser->IsInitialized() )
      {
        delete itsParser;
        return FALSE;
      }

      // Connect the signals which are interesting for us
      connect( itsParser, SIGNAL(ProjectKeyValue(QString &,QString &)), 
               SLOT(RestoreProject(QString &,QString &)) );
    
      // And start the action
      itsParser->StartParsing();

      // That's it for the Parser object
      delete itsParser;    

      itsParser = NULL;
    }
    
    DlgObject* obj;
    for( obj = itsObjects.first();
         obj;
         obj = itsObjects.next() )
    {
      if( !itsProjectWnd.IsAutoGenerating() )
      {
        obj->Show();
        //     itsProjectWnd.itsObjectList.Append( obj );
      }
	
      obj->Load();
    }
  }
  return TRUE;
}

bool Project::Save()
{
  QtArchGenerator generator;
  bool retVal = TRUE;
  DlgObject* obj;
    
  try
  {
    generator.WriteStart( itsFilename );

    generator.WriteStartBlock( "QtArch", TRUE, 
                               "type=\"Project\" version=\"2.1\"" );

    generator.WriteStartBlock( "Project", TRUE, "supertype=\"true\"" );
    
    generator.WriteSingleLine( "Name", itsName);

    generator.WriteStartList( "Objects" );
    for( obj = itsObjects.first();
         obj;
         obj = itsObjects.next() )
    {
      generator.WriteStartBlock( "Object" );
      generator.WriteSingleLine( "Type", obj->GetType() );
      generator.WriteSingleLine( "Name", obj->GetName() );
      generator.WriteEndBlock();
    }
    generator.WriteEndBlock();

    // End Project
    generator.WriteEndBlock();
    generator.WriteEndBlock();
    generator.WriteEnd();

  }
  catch( GeneratorError &err )
  {
      QMessageBox::warning( 
          0, 
          "QtArchitect - Error",
          "Error saving " + itsFilename + ": " + err.message );

    retVal = FALSE;
  }

  // Save all children
  for( obj = itsObjects.first();
       obj;
       obj = itsObjects.next() )
  {
    obj->Save();
  }
  
  status = DlgObject_Saved;
  emit Saved( this );
  needsSave = FALSE;
  
  return retVal;
}

void Project::Show()
{
}

void Project::Hide()
{
    DlgObject* obj;
    for( obj = itsObjects.first();
	 obj;
	 obj = itsObjects.next() )
    {
	obj->Hide();
    }
}

void Project::Raise()
{
}

void Project::Delete()
{
  // Tell all included objects to get deleted, as every single object will
  // remove itself from the list, always start at the first element
  DlgObject *obj;
  for( obj = itsObjects.first();
       obj;
       obj = itsObjects.first() )
  {
    obj->Delete();
  }

  // Remove yourself
  emit Deleted( this );

  // Delete yourself
  delete this;
  
}

bool Project::EnterProject( const Project *prj )
{
  return DlgObject::EnterProject( prj );
}

bool Project::LeaveProject( Project *prj )
{
  return DlgObject::LeaveProject( prj );
}

bool Project::InProject( const Project *prj )
{
  return DlgObject::InProject( prj );
}

void Project::CPAvailable( bool av )
{
  // shut up
  av = av;
}

void Project::AddToProject( DlgObject *newObject )
{
   itsObjects.append( newObject );
   status = DlgObject_Changed;
   needsSave = TRUE;
   emit Changed( this );
}

void Project::RemoveFromProject( DlgObject *obj )
{
   itsObjects.remove( obj );
   obj->Hide();
   status = DlgObject_Changed;
   needsSave = TRUE;
   emit Changed( this );
}

bool Project::Generate( bool aG )
{
  autoGenerate = aG;

  if( itsName.isEmpty() )
    Rename();

  GenerateAllObjects();

  // Get tmake and filename parameters
  TMakeDialog dlg( itsMakeFileName );
  dlg.exec();
  QString tmpl = dlg.GetTemplate();
  itsMakeFileName = dlg.GetMakefile();
  if( itsMakeFileName.isEmpty() )
    itsMakeFileName = "Makefile";

  // Parse real template name from tmpl 
  int pos = tmpl.findRev( '/' );
  if( pos != -1 )
    tmpl = tmpl.right( tmpl.length() - pos - 1 );
  pos = tmpl.findRev( '.' );
  if( pos != -1 )
    tmpl = tmpl.left( tmpl.length() - pos );

  // Write tmake project file
  QString itsBaseName = QFileInfo(itsName).baseName();
  QFile proFile( itsBaseName + ".pro" );

  // if project file exists, return. There may be changes by the user
  if(proFile.exists())
     return TRUE;
  
  if( ! proFile.open( IO_WriteOnly ) )
  {
    QMessageBox::warning( 0, tr( "Makefile generation" ), 
                          tr( "Couldn't open %1.pro for writing" ).
                          arg( itsMakeFileName ) );
    return FALSE;
  }

  QTextStream proStream( &proFile );
  proStream << "TEMPLATE\t= " << tmpl << endl;
  QListIterator<QString> iterHeader( genHeaders );
  if( iterHeader )
    proStream << "HEADERS \t= " << *(iterHeader.current());
  while( ++iterHeader ) 
    proStream << "\\\n\t   " << *(iterHeader.current());
  QListIterator<QString> iterSources( genSources );
  if( iterSources )
    proStream << "\nSOURCES \t= " << *(iterSources.current());
  while( ++iterSources )
    proStream << "\\\n\t   " << *(iterSources.current());
  proStream << "\nTARGET \t= " << itsBaseName << endl; 
  proFile.close();

  // call tmake 
  QString profName = itsBaseName + ".pro";
  SpawnTMake( itsMakeFileName, profName );

  // Edit generated Makefile
  MakeEdit makeEdit( itsMakeFileName );
  makeEdit.exec();

  return TRUE;
}

void Project::RestoreProject(QString &key, QString &value )
{
  if( key == "Name" )
  {
    itsName = value;
  } else if( key == "Dialog" )
  {
     Dialog *dialog = new Dialog( itsProjectWnd, "" );

     dialog->SetName( value );
     dialog->EnterProject( this );

     itsObjects.append( dialog );

     connect( dialog, SIGNAL(Changed(DlgObject*)), 
              itsProjectWnd.itsObjectList, SLOT(Update(DlgObject*)) );
     connect( dialog, SIGNAL(Saved(DlgObject*)),
              itsProjectWnd.itsObjectList, SLOT(Saved(DlgObject*)) );
     connect( dialog, SIGNAL(Closed(DlgObject*)), 
              SLOT(ObjClosed(DlgObject*)) );

     connect( dialog, SIGNAL(CPDataSet(DlgObject*)), 
              &itsProjectWnd, SLOT(NewCPBuffer(DlgObject*)) );
     connect( &itsProjectWnd, SIGNAL(CPBufferAvailable(bool)), dialog,
              SLOT(CPAvailable(bool)) );
  } else if( key == "Pixmap" )
  {
     Pixmap *pix = new Pixmap( itsProjectWnd, "" );

     pix->SetName( value );
     pix->EnterProject( this );

     itsObjects.append( pix );

     connect( pix, SIGNAL(Changed(DlgObject*)), 
              itsProjectWnd.itsObjectList, SLOT(Update(DlgObject*)) );
     connect( pix, SIGNAL(Saved(DlgObject*)),
              itsProjectWnd.itsObjectList, SLOT(Saved(DlgObject*)) );
     connect( pix, SIGNAL(Closed(DlgObject*)), 
              SLOT(ObjClosed(DlgObject*)) );

     connect( pix, SIGNAL(CPDataSet(DlgObject*)), 
              &itsProjectWnd, SLOT(NewCPBuffer(DlgObject*)) );
     connect( &itsProjectWnd, SIGNAL(CPBufferAvailable(bool)), pix,
              SLOT(CPAvailable(bool)) );
  }
}

void Project::RestoreXML( const XMLValue &value )
{
  // We can expect the value to be nested
  const XMLValueMap map = value.AsNested();

  itsName = map["Name"].AsQString();

  XMLValueList list = map["Objects"].AsList();
  QListIterator<XMLValue> it( list );
  for( ; it.current(); ++it )
  {
    const XMLValueMap map = it.current()->AsNested();

    if( map["Type"].AsQString() == "Dialog" )
    {
      Dialog *dialog = new Dialog( itsProjectWnd, "" );

      dialog->SetName( map["Name"].AsQString() );
      dialog->EnterProject( this );

      itsObjects.append( dialog );

      connect( dialog, SIGNAL(Changed(DlgObject*)), 
               itsProjectWnd.itsObjectList, SLOT(Update(DlgObject*)) );
      connect( dialog, SIGNAL(Saved(DlgObject*)),
               itsProjectWnd.itsObjectList, SLOT(Saved(DlgObject*)) );
      connect( dialog, SIGNAL(Closed(DlgObject*)), 
               SLOT(ObjClosed(DlgObject*)) );

      connect( dialog, SIGNAL(CPDataSet(DlgObject*)), 
               &itsProjectWnd, SLOT(NewCPBuffer(DlgObject*)) );
      connect( &itsProjectWnd, SIGNAL(CPBufferAvailable(bool)), dialog,
               SLOT(CPAvailable(bool)) );
    } else if( map["Type"].AsQString() == "Project" )
    {
      // Not implemented
    } else if( map["Type"].AsQString() == "Pixmap" )
    {
      Pixmap *pix = new Pixmap( itsProjectWnd, "" );

      pix->SetName( map["Name"].AsQString() );
      pix->EnterProject( this );

      itsObjects.append( pix );

      connect( pix, SIGNAL(Changed(DlgObject*)), 
               itsProjectWnd.itsObjectList, SLOT(Update(DlgObject*)) );
      connect( pix, SIGNAL(Saved(DlgObject*)),
               itsProjectWnd.itsObjectList, SLOT(Saved(DlgObject*)) );
      connect( pix, SIGNAL(Closed(DlgObject*)), 
               SLOT(ObjClosed(DlgObject*)) );

      connect( pix, SIGNAL(CPDataSet(DlgObject*)), 
               &itsProjectWnd, SLOT(NewCPBuffer(DlgObject*)) );
      connect( &itsProjectWnd, SIGNAL(CPBufferAvailable(bool)), pix,
               SLOT(CPAvailable(bool)) );
    }
  }
}

void Project::GenerateAllObjects()
{
  QString gSrc, gHdr, gMHdr;
  int pos;

  genSources.clear();
  genHeaders.clear();
  genMeta.clear();

  DlgObject* obj;
  for( obj = itsObjects.first();
       obj;
       obj = itsObjects.next() )
  {
    obj->Generate( autoGenerate );
    obj->GetGeneratedFiles( gSrc, gHdr, gMHdr );
    gSrc = gSrc.simplifyWhiteSpace();
    if( (pos = gSrc.find( ' ' )) != -1 )
    {
      genSources.append( new QString( gSrc.left( pos ) ));
      genSources.append( new QString( gSrc.right( gSrc.length() - pos ) ));
    } else
    {
      genSources.append( new QString( gSrc ) );
    }
    gHdr = gHdr.simplifyWhiteSpace();
    if( (pos = gHdr.find( ' ' )) != -1 )
    {
      genHeaders.append( new QString( gHdr.left( pos ) ));
      genHeaders.append( new QString( gHdr.right( gHdr.length() - pos ) ));
    } else
    {
      genHeaders.append( new QString( gHdr ) );
    }

    genMeta.append( new QString( gMHdr ) );
  }
}

void Project::GenerateMakeFile()
{
  makefile.clear();

  QFile templateMakefile( itsMakeFileTemplateName );

  if( ! templateMakefile.open( IO_ReadOnly ) )
  {
    QMessageBox::message( tr( "Error..." ), 
                          tr( "Unable to read template %1" ).
                          arg( itsMakeFileTemplateName ) );
    return;
  }

  QTextStream stream( &templateMakefile );
  while( ! stream.eof() )
  {
    QString line = stream.readLine();

    makefile.append( new QString( line ) );
    if( line == "# DlgEdit-Project: Do not remove this line" )
    {
      StringIter iSrc( genSources );
      if( iSrc.current() )
        makefile.append( new QString( "DLGSOURCES = \\" ) );
      while( iSrc.current() )
      {
        QString *str = iSrc;
        makefile.append( new QString( *str + (iSrc.atLast() ? "" : "\\") ));
        ++iSrc;
      }
      StringIter iHdr( genHeaders );
      if( iHdr.current() )
        makefile.append( new QString( "DLGHEADERS = \\" ));
      while( iHdr.current() )
      {
        QString *str = iHdr;
        makefile.append( new QString( *str + (iHdr.atLast() ? "" : "\\") ));
        ++iHdr;
      }
      StringIter iMHdr( genMeta );
      if( iMHdr.current() )
        makefile.append( new QString( "DLGMETA = \\" ));
      while( iMHdr.current() )
      {
        QString *str = iMHdr;
        makefile.append( new QString( *str + (iMHdr.atLast() ? "" : "\\") ));
        ++iMHdr;
      }
      makefile.append( new QString( "# DlgEdit-Project: End of generated section" ));
    }
  }
  templateMakefile.close();

}
void Project::ObjClosed( DlgObject *obj )
{
  ObjectListItem *item = 
    itsProjectWnd.itsObjectList->GetObjectListItem( obj, this );

  item->ObjectClosed( TRUE );
}


void Project::Rename()
{
  ProjectDialog dialog;

  dialog.SetName( itsName );

  if( dialog.exec() == QDialog::Accepted )
    itsName = dialog.GetName();
}

void Project::AddObject()
{
  QString str;

  str = QFileDialog::getOpenFileName();
    
  if( !str.isEmpty() )
  {
    // Most of the dirty work is done by ProjectWnd, it has to remember 
    // its loaded objects.
    DlgObject *obj = itsProjectWnd.ObjectLoad( str, this );

    AddToProject( obj );
  }
}

