/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: SelectedWidgetBorder.cpp,v 2.3 2000/07/23 20:36:43 jeffharris Exp $

*******************************************************************************/


#include <iostream.h>

#include <qapp.h>
#include <qcursor.h>
#include <qframe.h>
#include <qwidget.h>

#include "SelectedWidgetBorder.h"
#include "WidgetFrame.h"

const unsigned int SelectedWidgetBorder::myBorderWidth = 10;
const unsigned int SelectedWidgetBorder::myBorderLineWidth = 2;

SelectedWidgetBorder::SelectedWidgetBorder(
    QWidget*		theSelectedWidget,
    WidgetFrame*	theWidgetFrame,
    bool		isMoveable
    ) 
    :
    QObject( NULL ),
    mySelectedWidget( theSelectedWidget ),
    myWidgetFrame( theWidgetFrame ),
    isWidgetMoveable( isMoveable )
{
    mySelectedWidget->installEventFilter( this );

    unsigned int loc;
    for( loc = North; loc <= NorthWest; loc++ )
    {
        QFrame* frame = new QFrame( myWidgetFrame->parentWidget() );
        frame->installEventFilter( this );
        myBorderFrames.append( frame );

        updateBorderType( static_cast<BorderLocation> (loc) );

        frame->raise();
        frame->show();
    }

    myBorderFrames.setAutoDelete( TRUE );
    
    updateBorders();
}


SelectedWidgetBorder::~SelectedWidgetBorder()
{
}


void SelectedWidgetBorder::setMoveable(
    bool	isMoveable
    )
{
    isWidgetMoveable = isMoveable;
    
    unsigned int i;
    for( i = North; i <= NorthWest; ++i )
    {
        updateBorderType( static_cast<BorderLocation> (i) );
    }
}


bool SelectedWidgetBorder::eventFilter(
    QObject*	theSender,
    QEvent*	theEvent
    )
{
    switch( theEvent->type() )
    {
        case QEvent::MouseButtonPress:
        {
            if( theSender == mySelectedWidget )
            {
                break;
            }
	    
            QMouseEvent* mouseEvent = static_cast<QMouseEvent*>(theEvent);
            if( mouseEvent->button() == LeftButton )
            {	
                myDraggingPoint = mouseEvent->pos();
            }
	    
            break;
        }
        case QEvent::MouseMove:
        {
            if( theSender == mySelectedWidget )
            {
                break;
            }
	    
            QMouseEvent* mouseEvent = static_cast<QMouseEvent*>(theEvent);
	    
            QPoint delta = mouseEvent->pos() - myDraggingPoint;

            QRect widgetRect = mySelectedWidget->rect();
            widgetRect.moveBy( mySelectedWidget->x(),
                               mySelectedWidget->y() );

            QSize origWidgetSize = mySelectedWidget->size();

            // Calculate the new widget rect
            BorderLocation location = static_cast<BorderLocation>
                (myBorderFrames.findRef( static_cast<QFrame*>(theSender) ));

            widgetRect = calculateWidgetRect( widgetRect, 
                                              delta,
                                              location );
	    
            // Set the new position
            if( isWidgetMoveable )
            {
                mySelectedWidget->setGeometry( widgetRect );
            }
            else
            {
                if( widgetRect.width() != origWidgetSize.width() )
                {
                    mySelectedWidget->setFixedWidth( widgetRect.width() );
                }
		
                if( widgetRect.height() != origWidgetSize.height() )
                {
                    mySelectedWidget->setFixedHeight( widgetRect.height() );
                }
            }

            break;
        }
        case QEvent::Move:
        case QEvent::Resize:
        {
            if( theSender != mySelectedWidget )
            {
                break;
            }

            updateBorders();

            break;
        }
        default:
        {
            break;
        }
    }

    return FALSE;		// Don't stop event
}


void SelectedWidgetBorder::updateBorders()
{
    int widgetHeight = mySelectedWidget->height();
    int widgetWidth = mySelectedWidget->width();

    QPoint widgetPos = 
        myWidgetFrame->parentWidget()->mapFromGlobal( 
            mySelectedWidget->mapToGlobal( QPoint(0,0) ) );
    
    myBorderFrames.at(North)->setGeometry( widgetPos.x(), 
                                           widgetPos.y() - myBorderWidth,
                                           widgetWidth,
                                           myBorderWidth );

    myBorderFrames.at(South)->setGeometry( widgetPos.x(),
                                           widgetPos.y() + widgetHeight,
                                           widgetWidth,
                                           myBorderWidth );
    
    myBorderFrames.at(East)->setGeometry( widgetPos.x() + widgetWidth,
                                          widgetPos.y(),
                                          myBorderWidth,
                                          widgetHeight );
    
    myBorderFrames.at(West)->setGeometry( widgetPos.x() - myBorderWidth,
                                          widgetPos.y(),
                                          myBorderWidth,
                                          widgetHeight );

    myBorderFrames.at(NorthEast)->setGeometry( widgetPos.x() + widgetWidth,
                                               widgetPos.y() - myBorderWidth,
                                               myBorderWidth,
                                               myBorderWidth );
    
    myBorderFrames.at(NorthWest)->setGeometry( widgetPos.x() - myBorderWidth,
                                               widgetPos.y() - myBorderWidth,
                                               myBorderWidth,
                                               myBorderWidth );
    
    myBorderFrames.at(SouthEast)->setGeometry( widgetPos.x() + widgetWidth,
                                               widgetPos.y() + widgetHeight,
                                               myBorderWidth,
                                               myBorderWidth );
    
    myBorderFrames.at(SouthWest)->setGeometry( widgetPos.x() - myBorderWidth,
                                               widgetPos.y() + widgetHeight,
                                               myBorderWidth,
                                               myBorderWidth );
}


void SelectedWidgetBorder::updateBorderType(
    BorderLocation	theLocation
    )
{
    QFrame* frame = myBorderFrames.at( theLocation );

    switch( theLocation )
    {
        case North:
        case South:
        {
            frame->setFrameStyle( QFrame::HLine | QFrame::Raised );
            frame->setCursor( sizeVerCursor );
            break;
        }
        case West:
        case East:
        {
            frame->setFrameStyle( QFrame::VLine | QFrame::Raised );
            frame->setCursor( sizeHorCursor );
            break;
        }
        case NorthEast:
        case SouthWest:
        {
            frame->setFrameStyle( QFrame::Box | QFrame::Raised );
            frame->setCursor( sizeBDiagCursor );
		
            break;
        }
        case NorthWest:
        case SouthEast:
        {
            frame->setFrameStyle( QFrame::Box | QFrame::Raised );
            frame->setCursor( sizeFDiagCursor );
            break;
        }	    
    }

    QColor backgroundColor = frame->backgroundColor();

    switch( theLocation )
    {
        case North:
        case West:
        case NorthEast:
        case NorthWest:
        case SouthWest:
        {
            if( isWidgetMoveable )
            {
                frame->setPalette( QPalette( blue ) );
                frame->setLineWidth( myBorderLineWidth );
            }
            else
            {
                frame->setCursor( crossCursor );
                frame->setPalette( QPalette( darkBlue ) );
                frame->setLineWidth( 1 );
            }

            break;
        }
        case South:
        case East:
        case SouthEast:
        {
            frame->setPalette( QPalette( blue ) );
            frame->setLineWidth( myBorderLineWidth );
            break;
        }
    }

    frame->setBackgroundColor( backgroundColor );
}


QRect SelectedWidgetBorder::calculateWidgetRect(
    const QRect&	theWidgetRect,
    const QPoint&	theDelta,
    BorderLocation	theLocation
    ) const
{
    QRect newRect = theWidgetRect;

    int grid = myWidgetFrame->GetGrid();
	    
    switch( theLocation )
    {
        case North:
        {
            if( isWidgetMoveable )
            {
                newRect.setTop( 
                    minimum( newRect.bottom(),
                             round( newRect.top() + theDelta.y(), grid ) ) );
            }
		    
            break;
        }
        case South:
        {
            newRect.setBottom( 
                round( newRect.bottom() + theDelta.y(), grid ) - 1 );

            break;
        }
        case East:
        {
            newRect.setRight( 
                round( newRect.right() + theDelta.x(), grid ) - 1 );

            break;
        }
        case West:
        {
            if( isWidgetMoveable )
            {
                newRect.setLeft( 
                    minimum( newRect.right(),
                             round( newRect.left() + theDelta.x(), grid ) ) );
            }
		
            break;
        }
        case NorthEast:
        {
            if( isWidgetMoveable )
            {
                newRect.setTop( 
                    minimum( newRect.bottom(),
                             round( newRect.top() + theDelta.y(), grid ) ) );

                newRect.setRight( 
                    round( newRect.right() + theDelta.x(), grid ) - 1 );
            }
		
            break;
        }
        case NorthWest:
        {
            if( isWidgetMoveable )
            {
                newRect.setTop( 
                    minimum( newRect.bottom(),
                             round( newRect.top() + theDelta.y(), grid ) ) );

                newRect.setLeft( 
                    minimum( newRect.right(),
                             round( newRect.left() + theDelta.x(), grid ) ) );
            }
		    
            break;
        }
        case SouthEast:
        {
            newRect.setBottom( 
                round( newRect.bottom() + theDelta.y(), grid ) - 1 );
            newRect.setRight( 
                round( newRect.right() + theDelta.x(), grid ) - 1 );
		    
            break;
        }
        case SouthWest:
        {
            if( isWidgetMoveable )
            {
                newRect.setBottom( 
                    round( newRect.bottom() + theDelta.y(), grid ) - 1 );

                newRect.setLeft( 
                    minimum( newRect.right(),
                             round( newRect.left() + theDelta.x(), grid ) - 1 ) );
            }
		    
            break;
        }
        default:
        {
            break;
        }
    }


    // Cleanup the rect
    if( newRect.top() < 0 )
    {
        newRect.setTop( 0 );
    }
	    
    if( newRect.left() < 0 )
    {
        newRect.setLeft( 0 );
    }

    if( mySelectedWidget != myWidgetFrame )
    {
        if( newRect.bottom() > myWidgetFrame->height() )
        {
            newRect.setBottom( 
                round( myWidgetFrame->height(), grid ) - 1 );
        }
	    
        if( newRect.right() > myWidgetFrame->width() )
        {
            newRect.setRight(
                round( myWidgetFrame->width(), grid ) - 1 );
        }
    }
    else
    {
        if( newRect.bottom() < newRect.top() )
        {
            newRect.setBottom( newRect.top() );
        }
	
        if( newRect.right() < newRect.left() )
        {
            newRect.setRight( newRect.left() );
        }
    }
	    
    return newRect;
}
