/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: qembed.cpp,v 2.3 2000/07/23 20:36:43 jeffharris Exp $

*******************************************************************************/


#include <qstring.h>
#include <qfile.h>
#include <qlist.h>
#include <qtstream.h>
#include <qdatetm.h>

void    embedData( const QByteArray &input, QFile *output );
QString convertFileNameToCIdentifier( const char * );

int main( int argc, char **argv )
{
    if ( argc < 4 ) {
	warning( "Usage:\n\t%s header source files", argv[0] );
	return 1;
    }

    QFile source( argv[2] ), header( argv[1] );
    source.open( IO_WriteOnly );
    header.open( IO_WriteOnly );

    QTextStream sourceStream( &source );
    QTextStream headerStream( &header );

    headerStream << "#ifndef BITMAP_HEADER_INCLUDED" << endl;
    headerStream << "#define BITMAP_HEADER_INCLUDED" << endl << endl;

    sourceStream << "#include \"bmp/" << argv[1] << "\"\n";
    sourceStream << "#include \"bmp/PixmapFactory.h\"" << endl;

  // Embed data for all input files

    QString cname;
    QList<QString> bitmapNames;
    
    for ( int i=3; i<argc; i++ ) {
	QFile f( argv[i] );
	if ( !f.open(IO_ReadOnly) ) {
	    warning( "Cannot open file %s, ignoring it", argv[i] );
	    continue;
	}
	QByteArray a( f.size() );
	if ( f.readBlock(a.data(), f.size()) != (int)f.size() ) {
	    warning( "Cannot read file %s, ignoring it", argv[i] );
	    f.close();
	    continue;
	}

	cname = convertFileNameToCIdentifier( argv[i] );

	bitmapNames.append( new QString(cname) );

	headerStream << "extern const unsigned int " << cname << "_len;\n";
	headerStream << "extern const unsigned char " << cname << 
	    "_data[];\n\n";

	sourceStream << "const unsigned int " << cname << "_len = " <<
	    f.size() << ";\n";
	sourceStream << "const unsigned char " << cname << "_data[] = {";

	embedData( a, &source );
	sourceStream << "\n};\n\n";
	f.close();
    }

    QListIterator<QString> iter(bitmapNames);
    QString* bitmapName;

    // Generate bitmap enum
    headerStream << endl << "enum BitmapType" << endl;
    headerStream << '{' << endl;

    while( (bitmapName = iter()) )
    {
	headerStream << "    " << *bitmapName;
	
	// If not at the last item, put a comma
	if( iter.current() )
	{
	    headerStream << ',' << endl;
	}
	else
	{
	    headerStream << endl;
	}
    }
    
    headerStream << "};" << endl;

    headerStream << endl << "#endif // BITMAP_HEADER_INCLUDED" << endl;

    // Create the definition of the PixmapFactory class method to convert
    // the bitmap type to the length and data values.
    sourceStream << endl;
    sourceStream << "void PixmapFactory::GetBitmapData( BitmapType            theBitmapType," << endl;
    sourceStream << "                                   unsigned int&         theBitmapLength," << endl;
    sourceStream << "                                   const unsigned char*& theBitmapData )" << endl;
    sourceStream << "{" << endl;
    sourceStream << "    theBitmapLength = 0;" << endl;
    sourceStream << "    theBitmapData = 0;" << endl;
    sourceStream << endl;
    sourceStream << "    switch( theBitmapType )" << endl;
    sourceStream << "    {" << endl;

    iter.toFirst();
    while( (bitmapName = iter()) )
    {
	sourceStream << "        case " << *bitmapName << ':' << endl;
	sourceStream << "        {" << endl;
	sourceStream << "            theBitmapLength = " << *bitmapName <<
	    "_len;" << endl;
	sourceStream << "            theBitmapData = " << *bitmapName <<
	    "_data;" << endl;
	sourceStream << "            break;" << endl;
	sourceStream << "        }" << endl;
    }
    
    sourceStream << "        default:" << endl;
    sourceStream << "            break;" << endl;
    sourceStream << "    }" << endl;
    sourceStream << "}" << endl;

    return 0;
}


QString convertFileNameToCIdentifier( const char *s )
{
    QString r = s;
    int len = r.length();
    if ( len > 0 && !r[0].isLetter() )
	r[0] = '_';
    for ( int i=1; i<len; i++ ) {
	if ( !(r[i].isLetter() || r[i].isNumber()) )
	    r[i] = '_';
    }
    return r;
}


void embedData( const QByteArray &input, QFile *output )
{
    static char hexdigits[] = "0123456789abcdef";
    int nbytes = input.size();
    char s[100];
    char* p = s;
    for ( int i=0; i<nbytes; i++ ) {
	if ( (i%14) == 0 ) {
	    strcpy( p, "\n    " );
	    output->writeBlock( s, strlen(s) );
	    p = s;
	}
	int v = (int)((uchar)input[i]);
	*p++ = '0';
	*p++ = 'x';
	*p++ = hexdigits[(v >> 4) & 15];
	*p++ = hexdigits[v & 15];
	if ( i < nbytes-1 )
	    *p++ = ',';
    }
    if ( p > s ) {
	*p = '\0';
	output->writeBlock( s, strlen(s) );
    }
}
