/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: BoxLayout.cpp,v 2.5 2000/07/23 20:36:43 jeffharris Exp $

*******************************************************************************/


#include "layout/BoxLayout.h"
#include "layout/BoxLayoutPropPage.h"
#include "layout/LayoutHelper.h"

#include "widget/DlgMenuBar.h"
#include "widget/DlgWidget.h"

#include "xml/QtArchGenerator.h"

#define Inherited LayoutItem

BoxLayout::BoxLayout(
    LayoutItem*	theParent
    )
    :
    Inherited( LayoutItem::LI_BoxLayout, 
	       theParent,
	       GetTreeLabel(), 
	       &PixmapFactory::GetPixmap( BoxLayout_bmp ) ),
    myDirection( QBoxLayout::Down ),
    myBorder( 5 ),
    myAutoBorder( 5 ),
    myStrut( 0 ),
    myStretch( 1 ),
    myMenuBarWidget( NULL )
{
}


BoxLayout::BoxLayout(
    QListView*	theParent
    )
    :
    Inherited( LayoutItem::LI_BoxLayout, 
	       theParent,
	       GetTreeLabel(), 
	       &PixmapFactory::GetPixmap( BoxLayout_bmp ) ),
    myDirection( QBoxLayout::Down ),
    myBorder( 5 ),
    myAutoBorder( 5 ),
    myStrut( 0 ),
    myStretch( 1 ),
    myMenuBarWidget( NULL )
{
}


BoxLayout::~BoxLayout()
{
}


LayoutItemPropPage* BoxLayout::GetPropertyPage( QWidget*	parent,
						bool		topLevel,
						const WidgetLayout&	theWidgetLayout )
{
    return new BoxLayoutPropPage( this, topLevel, theWidgetLayout, parent );
}


QLayout* BoxLayout::RecreateLayout( QWidget*	theBaseWindow )
{
    QBoxLayout* layout = new QBoxLayout( theBaseWindow,
					 myDirection,
					 myBorder,
					 myAutoBorder,
					 myName );

    SetLayoutContents( layout );
    
    return layout;
}


void BoxLayout::AddToLayout( QBoxLayout*	theBoxLayout )
{
    QBoxLayout* layout = new QBoxLayout( myDirection,
					 myAutoBorder,
					 myName );

    theBoxLayout->addLayout( layout, myStretch );

    SetLayoutContents( layout );
}


void BoxLayout::AddToLayout( QGridLayout*	theGridLayout,
			     int		theRowNum,
			     int		theColNum )
{
    QBoxLayout* layout = new QBoxLayout( myDirection,
					 myAutoBorder,
					 myName );
    
    theGridLayout->addLayout( layout, theRowNum, theColNum );
    
    SetLayoutContents( layout );
}


void BoxLayout::SaveContents( QtArchGenerator&	generator ) const
{
  generator.WriteStartBlock( Type2String() );

  QString direction;
  switch( myDirection )
  {
    case QBoxLayout::LeftToRight: direction = "LeftToRight"; break;
    case QBoxLayout::RightToLeft: direction = "RightToLeft"; break;
    case QBoxLayout::TopToBottom: direction = "TopToBottom"; break;
    case QBoxLayout::BottomToTop: direction = "BottomToTop"; break;
  }
  generator.WriteSingleLine( "Direction", direction );

  generator.WriteXMLValue( "Border", myBorder );
  generator.WriteXMLValue( "AutoBorder", myAutoBorder );
  generator.WriteXMLValue( "Strut", myStrut );
  generator.WriteXMLValue( "Stretch", myStretch );
  generator.WriteSingleLine( "Name", myName );

  if( myMenuBarWidget )
  {
    generator.WriteSingleLine( "Menu", myMenuBarWidget->GetWidgetName() );
  }

  SaveChildren( generator );

  // BoxLayout
  generator.WriteEndBlock();
}


bool BoxLayout::RestoreKeyValue( const QString&		theKey,
				 const QString&		theValue,
				 const DlgWidget*	theDlgWidget )
{
  bool rc = TRUE;
    
  if( theKey == "Direction" )
  {
    QBoxLayout::Direction dir;
	
    if( theValue == "TopToBottom" )
    {
	    dir = QBoxLayout::TopToBottom;
    }
    else if( theValue == "BottomToTop" )
    {
	    dir = QBoxLayout::BottomToTop;
    }
    else if( theValue == "LeftToRight" )
    {
	    dir = QBoxLayout::LeftToRight;
    }
    else if( theValue == "RightToLeft" )
    {
	    dir = QBoxLayout::RightToLeft;
    }
    else
    {
	    rc = FALSE;
    }
	
    if( rc )
    {
	    myDirection = dir;
    }
  }
  else if( theKey == "Border" )
  {
    myBorder = theValue.toInt();
  }
  else if( theKey == "AutoBorder" )
  {
    myAutoBorder = theValue.toInt();
  }
  else if( theKey == "Name" )
  {
    myName = theValue;
  }
  else if( theKey == "Strut" )
  {
    myStrut = theValue.toInt();
  }
  else if( theKey == "Stretch" )
  {
    myStretch = theValue.toInt();
  }
  else if( theKey == "Menu" )
  {
    if( theDlgWidget && 
        (theDlgWidget->GetType() == DlgWidget::DW_MenuBar) )
    {
	    myMenuBarWidget = static_cast<const DlgMenuBar*>
        (theDlgWidget);
    }
    else
    {
	    rc = FALSE;
    }
  }
  else
  {
    rc = FALSE;
  }

  return rc;
}

void BoxLayout::Restore( const XMLValue &value, LayoutHelper *helper )
{
  // Sanity check
  if( value.GetTypeString() != Type2String() )
    return;

  const XMLValueMap map = value.AsNested();

  QString dirStr = map["Direction"].AsQString();
	
  if( dirStr == "TopToBottom" )
  {
    myDirection = QBoxLayout::TopToBottom;
  }
  else if( dirStr == "BottomToTop" )
  {
    myDirection = QBoxLayout::BottomToTop;
  }
  else if( dirStr == "LeftToRight" )
  {
    myDirection = QBoxLayout::LeftToRight;
  }
  else if( dirStr == "RightToLeft" )
  {
    myDirection = QBoxLayout::RightToLeft;
  }

  myBorder = map["Border"].AsInt();
  myAutoBorder = map["AutoBorder"].AsInt();
  myName = map["Name"].AsQString();
  myStrut = map["Strut"].AsInt();
  myStretch = map["Stretch"].AsInt();
  myMenuBarWidget = helper->GetMenuWidget( map["Menu"].AsQString() );

  RestoreChildren( map["Children"], helper );
}

void BoxLayout::GenerateSource( QTextStream&	theStream,
				const QString&	theLayoutId,
				const QString&	theLayoutParentWidget ) const
{
    // Just call GenerateSourceInBox with the parameters for a top level
    // layout
    GenerateSourceInBox( theStream, "", theLayoutId, theLayoutParentWidget );
}


void BoxLayout::GenerateSourceInBox( QTextStream&	theStream,
				     const QString&	theLayoutVarName,
				     const QString&	theLayoutId,
				     const QString&	theLayoutParentWidget ) const
{
    QString varName = "qtarch_layout_" + theLayoutId;

    bool topLevel = theLayoutVarName.isEmpty();

    GenerateCtor( theStream, varName, topLevel, theLayoutParentWidget );

    if( !topLevel )
    {
	// Add this layout to its parent
	theStream << '\t' << theLayoutVarName << "->addLayout( " <<
	    varName << ", " << myStretch << " );\n";
    }

    GenerateContents( theStream, varName, theLayoutId, theLayoutParentWidget );
}


void BoxLayout::GenerateSourceInGrid( QTextStream&	theStream,
				      const QString&	theLayoutVarName,
				      const QString&	theLayoutId,
				      unsigned int	theRowNum,
				      unsigned int	theColNum,
				      const QString&	theLayoutParentWidget ) const
{
    QString varName = "qtarch_layout_" + theLayoutId;

    bool topLevel = theLayoutVarName.isEmpty();

    GenerateCtor( theStream, varName, topLevel, theLayoutParentWidget );

    if( !topLevel )
    {
	// Add this layout to its parent
	theStream << '\t' << theLayoutVarName << "->addLayout( " <<
	    varName << ", " << theRowNum << ", " << theColNum << " );\n";
    }

    GenerateContents( theStream, varName, theLayoutId, theLayoutParentWidget );
}


void BoxLayout::SetName( const QString&	name )
{
    myName = name;

    QString label = GetTreeLabel().copy();
    if( !name.isEmpty() )
    {
	label += " (" + name + ")";
    }
    
    setText( 0, label );
}

    
void BoxLayout::SetLayoutContents( QBoxLayout*	theLayout )
{
    theLayout->addStrut( myStrut );
    
    if( myMenuBarWidget )
    {
	theLayout->setMenuBar( 
	    static_cast<QMenuBar*> (myMenuBarWidget->GetWidget()) );
    }
    
    LayoutItem* child = 0;
    for( child = static_cast<LayoutItem*>(firstChild());
	 child;
	 child = static_cast<LayoutItem*>(child->nextSibling()) )
    {
	child->AddToLayout( theLayout );
    }
}

    
void BoxLayout::GenerateCtor( QTextStream&	theStream,
			      const QString&	theVarName,
			      bool		isTopLevel,
			      const QString&	theLayoutParentWidget ) const
{
    theStream << "\tQBoxLayout* " << theVarName << " = new QBoxLayout( ";
    
    if( isTopLevel )
    {
	if( theLayoutParentWidget.isEmpty() )
	{
	    theStream << "this, ";
	}
	else
	{
	    theStream << theLayoutParentWidget << ", ";
	}
    }
    
    theStream << "QBoxLayout::";
    switch( myDirection )
    {
	case QBoxLayout::TopToBottom:
	{
	    theStream << "TopToBottom";
	    break;
	}
	case QBoxLayout::BottomToTop:
	{
	    theStream << "BottomToTop";
	    break;
	}
	case QBoxLayout::LeftToRight:
	{
	    theStream << "LeftToRight";
	    break;
	}
	case QBoxLayout::RightToLeft:
	{
	    theStream << "RightToLeft";
	    break;
	}
    }
    theStream << ", ";
    
    if( isTopLevel )
    {
	theStream << myBorder << ", ";
    }
    
    theStream << myAutoBorder << ", ";
    
    QString name = "NULL";
    if( !myName.isEmpty() )
    {
	name = "\"" + myName + "\"";
    }
    theStream << name << " );\n";
}


void BoxLayout::GenerateContents( QTextStream&		theStream,
				  const QString&	theVarName,
				  const QString&	theLayoutId,
				  const QString&	theLayoutParentWidget ) const
{
    theStream << '\t' << theVarName << "->addStrut( " << myStrut << " );\n";
    
    if( myMenuBarWidget )
    {
	theStream << '\t' << theVarName << "->setMenuBar( " <<
	    myMenuBarWidget->GetGeneratedVarName() << " );\n";
    }

    // Generate the children
    LayoutItem* item;
    int pos;
    QString idStr;

    for( item = static_cast<LayoutItem*>(firstChild()), pos = 1;
	 item;
	 item = static_cast<LayoutItem*>(item->nextSibling()), ++pos )
    {
	idStr = theLayoutId + '_' + idStr.setNum(pos);
	
	item->GenerateSourceInBox( theStream, 
				   theVarName,
				   idStr,
				   theLayoutParentWidget );
    }
}


QString BoxLayout::GetTreeLabel()
{
    static QString treeLabel = QObject::tr( "Box Layout" );
    return treeLabel;
}
