/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: GridLayout.cpp,v 2.5 2000/07/23 20:36:43 jeffharris Exp $

*******************************************************************************/


#include "layout/GridLayout.h"
#include "layout/GridLayoutPropPage.h"
#include "layout/GridRow.h"
#include "layout/LayoutHelper.h"

#include "widget/DlgMenuBar.h"

#include "xml/QtArchGenerator.h"

#define Inherited LayoutItem

GridLayout::GridLayout(
    LayoutItem*	theParent
    )
    :
    Inherited( LayoutItem::LI_GridLayout, 
	       theParent,
	       GetTreeLabel(), 
	       &PixmapFactory::GetPixmap( GridLayout_bmp ) ),
    myBorder( 5 ),
    myAutoBorder( 5 ),
    myMenuBarWidget( NULL ),
    myBoxStretch( 1 )
{
    myColumnInfo.setAutoDelete( TRUE );
}


GridLayout::GridLayout(
    QListView*	theParent
    )
    :
    Inherited( LayoutItem::LI_GridLayout, 
	       theParent,
	       GetTreeLabel(), 
	       &PixmapFactory::GetPixmap( GridLayout_bmp ) ),
    myBorder( 5 ),
    myAutoBorder( 5 ),
    myMenuBarWidget( NULL ),
    myBoxStretch( 1 )
{
    myColumnInfo.setAutoDelete( TRUE );
}


GridLayout::~GridLayout()
{
}


LayoutItemPropPage* GridLayout::GetPropertyPage( QWidget*	parent,
						 bool		topLevel,
						 const WidgetLayout&	theWidgetLayout )
{
    return new GridLayoutPropPage( this, topLevel, theWidgetLayout, parent );
}


void GridLayout::UpdateColumnInfo()
{
    // Determine the number of columns
    int numColumns = 0;
    QListViewItem* row;
    for( row = firstChild(); row; row = row->nextSibling() )
    {
	if( row->childCount() > numColumns )
	{
	    numColumns = row->childCount();
	}
    }
    
    // Add any extra column infos
    QListIterator<GridLayout::GridColumnInfo> iter(myColumnInfo);
    GridLayout::GridColumnInfo* columnInfo;
    int i;
    for( i = 0, columnInfo = iter(); 
	 i < numColumns; 
	 ++i, columnInfo = iter() )
    {
	if( !columnInfo )
	{
	    // Need to add a new column info
	    columnInfo = new GridLayout::GridColumnInfo( 5,1 );
	    myColumnInfo.append( columnInfo );
	}
    }
    
    // Remove any extra column infos
    int numColInfos = myColumnInfo.count();
    for( i = numColumns; i < numColInfos; ++i )
    {
	myColumnInfo.removeLast();
    }
}


QLayout* GridLayout::RecreateLayout( QWidget*	theBaseWindow )
{
    UpdateColumnInfo();

    unsigned int numRows = childCount();
    unsigned int numCols = myColumnInfo.count();

    QGridLayout* layout = new QGridLayout( theBaseWindow,
					   numRows,
					   numCols,
					   myBorder,
					   myAutoBorder,
					   myName );
    
    SetLayoutContents( layout );
    
    return layout;
}


void GridLayout::AddToLayout( QBoxLayout*	theBoxLayout )
{
    UpdateColumnInfo();

    unsigned int numRows = childCount();
    unsigned int numCols = myColumnInfo.count();

    QGridLayout* layout = new QGridLayout( numRows,
					   numCols,
					   myAutoBorder,
					   myName );

    theBoxLayout->addLayout( layout, myBoxStretch );
    
    SetLayoutContents( layout );
}


void GridLayout::SaveContents( QtArchGenerator &generator ) const
{
  generator.WriteStartBlock( Type2String() );

  generator.WriteXMLValue( "Border", myBorder );
  generator.WriteXMLValue( "AutoBorder", myAutoBorder );
  generator.WriteSingleLine( "Name", myName );

  if( myMenuBarWidget )
  {
    generator.WriteSingleLine( "Menu", myMenuBarWidget->GetWidgetName() );
  }

  // Save the column info
  QListIterator<GridLayout::GridColumnInfo> iter( myColumnInfo );
  GridColumnInfo* info;

  generator.WriteStartList( "ColumnList" );
  while( (info = iter()) )
  {
    generator.WriteStartBlock( "ColumnInfo" );
    generator.WriteXMLValue( "Spacing", info->mySpacing );
    generator.WriteXMLValue( "Stretch", info->myStretch );
    generator.WriteEndBlock();
  }
  generator.WriteEndBlock();
  
  generator.WriteXMLValue( "BoxStretch", myBoxStretch );

  SaveChildren( generator );

  generator.WriteEndBlock();
}


bool GridLayout::RestoreKeyValue( const QString&	theKey,
				  const QString&	theValue,
				  const DlgWidget*	theDlgWidget )
{
    bool rc = TRUE;

    if( theKey == "Border" )
    {
	myBorder = theValue.toInt();
    }
    else if( theKey == "AutoBorder" )
    {
	myAutoBorder = theValue.toInt();
    }
    else if( theKey == "Name" )
    {
	myName = theValue;
    }
    else if( theKey == "Menu" )
    {
	if( theDlgWidget &&
	    (theDlgWidget->GetType() == DlgWidget::DW_MenuBar) )
	{
	    myMenuBarWidget = 
		static_cast<const DlgMenuBar*> (theDlgWidget);
	}
	else
	{
	    rc = FALSE;
	}
    }
    else if( theKey == "ColumnInfo" )
    {
	int pos = theValue.find( ' ' );
	if( pos != -1 )
	{
	    QString spacing = theValue.left( pos ).stripWhiteSpace();
	    QString stretch = 
		theValue.mid( pos + 1, 
			      theValue.length() - pos - 1 ).stripWhiteSpace();

	    GridColumnInfo* info = new GridColumnInfo( spacing.toInt(),
						       stretch.toInt() );
	    myColumnInfo.append( info );
	}
	else
	{
	    rc = FALSE;
	}
    }
    else if( theKey == "BoxStretch" )
    {
	myBoxStretch = theValue.toInt();
    }
    else
    {
	rc = FALSE;
    }
    
    return rc;
}

void GridLayout::Restore( const XMLValue &value, LayoutHelper *helper )
{
  // Sanity check
  if( value.GetTypeString() != Type2String() )
    return;

  const XMLValueMap map = value.AsNested();
  
  myBorder = map["Border"].AsInt();
  myAutoBorder = map["AutoBorder"].AsInt();
  myName = map["Name"].AsQString();
  myMenuBarWidget = helper->GetMenuWidget( map["Menu"].AsQString() );
  myBoxStretch = map["BoxStretch"].AsInt();

  XMLValueList list = map["ColumnList"].AsList();
  QListIterator<XMLValue> it( list );
  for( ; it.current(); ++it )
  {
    const XMLValueMap map = it.current()->AsNested();

    GridColumnInfo* info = new GridColumnInfo( map["Spacing"].AsInt(),
                                               map["Stretch"].AsInt() );
    myColumnInfo.append( info );
  }
  
  RestoreChildren( map["Children"], helper );
}


void GridLayout::GenerateSource( QTextStream&	theStream,
				 const QString&	theLayoutId,
				 const QString&	theLayoutParentWidget ) const
{
    // Just call GenerateSourceInBox with the parameters for a top level
    // layout
    GenerateSourceInBox( theStream, "", theLayoutId, theLayoutParentWidget );
}


void GridLayout::GenerateSourceInBox( QTextStream&	theStream,
				      const QString&	theLayoutVarName,
				      const QString&	theLayoutId,
				      const QString&	theLayoutParentWidget ) const
{
    QString varName = "qtarch_layout_" + theLayoutId;

    bool topLevel = theLayoutVarName.isEmpty();

    GenerateCtor( theStream, varName, topLevel, theLayoutParentWidget );

    if( !topLevel )
    {
	// Add this layout to its parent
	theStream << '\t' << theLayoutVarName << "->addLayout( " <<
	    varName << ", " << myBoxStretch << " );\n";
    }

    GenerateContents( theStream, varName, theLayoutId, theLayoutParentWidget );
}


void GridLayout::GenerateSourceInGrid( QTextStream&	theStream,
				       const QString&	theLayoutVarName,
				       const QString&	theLayoutId,
				       unsigned int	theRowNum,
				       unsigned int	theColNum,
				       const QString&	theLayoutParentWidget ) const
{
    QString varName = "qtarch_layout_" + theLayoutId;

    bool topLevel = theLayoutVarName.isEmpty();

    GenerateCtor( theStream, varName, topLevel, theLayoutParentWidget );

    if( !topLevel )
    {
	// Add this layout to its parent
	theStream << '\t' << theLayoutVarName << "->addLayout( " <<
	    varName << ", " << theRowNum << ", " << theColNum << " );\n";
    }

    GenerateContents( theStream, varName, theLayoutId, theLayoutParentWidget );
}


void GridLayout::SetName( const QString&	name )
{
    myName = name;

    QString label = GetTreeLabel().copy();
    if( !name.isEmpty() )
    {
	label += " (" + name + ")";
    }
    
    setText( 0, label );
}


void GridLayout::SetLayoutContents( QGridLayout*	theLayout )
{
    // Loop through all the rows
    GridRow* item;
    int rowNum;

    for( item = static_cast<GridRow*>(firstChild()), rowNum = 0; 
	 item; 
	 item = static_cast<GridRow*>(item->nextSibling()), ++rowNum )
    {
	// Add all the columns to the row
	item->AddToLayout( theLayout, rowNum );
    }

    // Set the properties on the columns
    QListIterator<GridLayout::GridColumnInfo> iter(myColumnInfo);
    GridLayout::GridColumnInfo* columnInfo;
    int colNum = 0;
    
    while( (columnInfo = iter()) )
    {
	theLayout->setColStretch( colNum, columnInfo->myStretch );
	theLayout->addColSpacing( colNum, columnInfo->mySpacing );
	
	++colNum;
    }
}


void GridLayout::GenerateCtor( QTextStream&	theStream,
			       const QString&	theVarName,
			       bool		isTopLevel,
			       const QString&	theLayoutParentWidget ) const
{
    theStream << "\tQGridLayout* " << theVarName << " = new QGridLayout( ";
    
    if( isTopLevel )
    {
	if( theLayoutParentWidget.isEmpty() )
	{
	    theStream << "this, ";
	}
	else
	{
	    theStream << theLayoutParentWidget << ", ";
	}
    }
    
    // Number of rows
    theStream << childCount() << ", ";

    // Number of columns
    theStream << myColumnInfo.count() << ", ";
    
    if( isTopLevel )
    {
	theStream << myBorder << ", ";
    }
    
    theStream << myAutoBorder << ", ";
    
    QString name = "NULL";
    if( !myName.isEmpty() )
    {
	name = "\"" + myName + "\"";
    }
    theStream << name << " );\n";
}


void GridLayout::GenerateContents( QTextStream&		theStream,
				   const QString&	theVarName,
				   const QString&	theLayoutId,
				   const QString&	theLayoutParentWidget ) const
{
    if( myMenuBarWidget )
    {
	theStream << '\t' << theVarName << "->setMenuBar( " <<
	    myMenuBarWidget->GetGeneratedVarName() << " );\n";
    }

    // Set the info for all the columns
    QListIterator<GridLayout::GridColumnInfo> iter( myColumnInfo );
    GridLayout::GridColumnInfo* info;
    int pos = 0;
    
    while( (info = iter()) )
    {
	theStream << '\t' << theVarName << "->addColSpacing( " <<
	    pos << ", " << info->mySpacing << " );\n";
	theStream << '\t' << theVarName << "->setColStretch( " <<
	    pos << ", " << info->myStretch << " );\n";

	++pos;
    }

    // Generate the children
    LayoutItem* item;
    QString idStr;

    for( item = static_cast<LayoutItem*>(firstChild()), pos = 1; 
	 item; 
	 item = static_cast<LayoutItem*>(item->nextSibling()), ++pos )
    {
	idStr = theLayoutId + '_' + idStr.setNum(pos);
	
	// Its children should be grid rows.  They need to know what row
	// they represent.  They will fill in the column number before
	// calling their children.
	item->GenerateSourceInGrid( theStream, 
				    theVarName,
				    idStr,
				    pos - 1,
				    0,
				    theLayoutParentWidget );
    }
}


QString GridLayout::GetTreeLabel()
{
    static QString treeLabel = QObject::tr( "Grid Layout" );
    return treeLabel;
}


GridLayout::GridColumnInfo::GridColumnInfo( int	spacing,
					    int	stretch ) :
    mySpacing( spacing ),
    myStretch( stretch )
{
}


GridLayout::GridColumnInfo::~GridColumnInfo()
{
}


