/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: GridLayoutPropPage.cpp,v 2.5 2000/07/23 20:36:43 jeffharris Exp $

*******************************************************************************/

#include <qregexp.h>
#include <qstring.h>
#include <qstrlist.h>

#include "layout/GridLayoutPropPage.h"
#include "layout/GridLayout.h"
#include "WidgetLayout.h"
#include "widget/DlgMenuBar.h"

#define Inherited GridLayoutPropPageData

GridLayoutPropPage::GridLayoutPropPage
(
    GridLayout*		layout,
    bool		topLevel,
    const WidgetLayout&	theWidgetLayout,
    QWidget* 		parent,
    const char*		name
)
    :
    Inherited( parent, name ),
    myLayout( layout ),
    myWidgetLayout( theWidgetLayout )
{
    QString str;

    myBorder->setText( str.setNum(myLayout->GetBorder()) );
    myBorder->setEnabled( topLevel );
    myBorderLabel->setEnabled( topLevel );

    myAutoBorder->setText( str.setNum(myLayout->GetAutoBorder()) );

    myName->setText( myLayout->GetName() );
    
    // Insert names of menu bars into combo box
    QStrList menus;
    myWidgetLayout.GetWidgetTypeNames( DlgWidget::DW_MenuBar, menus );

    myMenuBars->insertItem( tr( "(None)" ) );
    myMenuBars->insertStrList( &menus );

    QString menuName;
    if( myLayout->GetMenuBarWidget() )
    {
	menuName = myLayout->GetMenuBarWidget()->GetWidgetName();
    }
    
    int index = 0;

    if( !menuName.isEmpty() )
    {
	index = menus.find( menuName );
	if( index == -1 )
	{
	    index = 0;
	}
	else
	{
	    // account for the "(None)" entry
	    ++index;
	}
    }
    
    myMenuBars->setCurrentItem( index );

    // Fill column list box and synchronize column info
    myLayout->UpdateColumnInfo();
    QList<GridLayout::GridColumnInfo>& columnInfo = myLayout->GetColumnInfo();

    if( columnInfo.count() > 0 )
    {
	QListIterator<GridLayout::GridColumnInfo> columnIter( columnInfo );
	GridLayout::GridColumnInfo* info = NULL;
	int i = 1;

	while( (info = columnIter()) )
	{
	    QString line;
	    line.sprintf( "%d - %d, %d", 
			  i, 
			  info->mySpacing, 
			  info->myStretch );
	    myColumns->insertItem( line );

	    ++i;
	}
    }
    else
    {
	myColGroupBox->setEnabled( FALSE );
	myColumns->setEnabled( FALSE );
    }

    // Box Stretch
    myBoxStretch->setText( str.setNum( myLayout->GetBoxStretch() ) );
    
    bool parentIsBox =
	(myLayout->GetParentType() == LayoutItem::LI_BoxLayout);
    myBoxStretch->setEnabled( parentIsBox );
    myBoxStretchLabel->setEnabled( parentIsBox );
    
    UpdateScreen();

    myChangedFlag = FALSE;
}


GridLayoutPropPage::~GridLayoutPropPage()
{
}


bool GridLayoutPropPage::Apply()
{
    if( myLayout )
    {
	QString str;
    
	str = myBorder->text();
	myLayout->SetBorder( str.toInt() );
    
	str = myAutoBorder->text();
	myLayout->SetAutoBorder( str.toInt() );
    
	myLayout->SetName( myName->text() );

	str = "";
	if( myMenuBars->currentItem() > 0 )
	{
	    str = myMenuBars->currentText();
	}
	myLayout->SetMenuBarWidget( 
	    (const DlgMenuBar*)myWidgetLayout.GetWidget(str) );

	QList<GridLayout::GridColumnInfo>& columnInfo = 
	    myLayout->GetColumnInfo();
	QListIterator<GridLayout::GridColumnInfo> columnIter( columnInfo );
	GridLayout::GridColumnInfo* info;
	
	int numCols = myColumns->count();
	int i;
	for( i = 0, info = columnIter(); i < numCols; 
	     i++, info = columnIter() )
	{
	    ParseColumnLine( myColumns->text( i ),
			     info->mySpacing,
			     info->myStretch );
	}

	// Box Stretch
	str = myBoxStretch->text();
	myLayout->SetBoxStretch( str.toInt() );
    }

    return myChangedFlag;
}


void GridLayoutPropPage::ColumnSelected()
{
    UpdateScreen();

    int spacing, stretch;
    ParseColumnLine( myColumns->currentText(), 
		     spacing, 
		     stretch );
    
    QString str;
    
    myColSpacing->setText( str.setNum( spacing ) );
    myColStretch->setText( str.setNum( stretch ) );
}


void GridLayoutPropPage::SetColumnInfo()
{
    QString str;

    str = myColSpacing->text();
    int spacing = str.toInt();
    
    str = myColStretch->text();
    int stretch = str.toInt();
    
    str.sprintf( "%d - %d, %d", 
		 myColumns->currentItem() + 1, 
		 spacing, 
		 stretch );
    myColumns->changeItem( str, myColumns->currentItem() );
}


void GridLayoutPropPage::UpdateScreen()
{
    bool colSelected = myColumns->currentItem() >= 0;
    
    myColSpacing->setEnabled( colSelected );
    myColSpacingLabel->setEnabled( colSelected );
    myColStretch->setEnabled( colSelected );
    myColStretchLabel->setEnabled( colSelected );
    myColumnGB->setEnabled( colSelected );
    mySetColInfoButton->setEnabled( colSelected );
}


void GridLayoutPropPage::ParseColumnLine( const QString&        line,
					  int&                  spacing,
					  int&                  stretch )
{
    QString str = line;

    // Remove column number from string
    str.replace( QRegExp("^[0-9]+ - "), "" );
	    
    // Separate spacing and stretch
    int pos = str.find(',');
    QString spacingStr = str.left( pos );
    QString stretchStr = str.mid( pos + 1, str.length() );

    spacing = spacingStr.stripWhiteSpace().toInt();
    stretch = stretchStr.stripWhiteSpace().toInt();
}
