/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: LayoutItem.cpp,v 2.5 2000/07/23 20:36:43 jeffharris Exp $

*******************************************************************************/


#include "layout/LayoutItem.h"

#include "layout/BoxLayout.h"
#include "layout/BoxSpacing.h"
#include "layout/BoxStretch.h"
#include "layout/GridLayout.h"
#include "layout/GridRow.h"
#include "layout/GridSpacer.h"
#include "layout/LayoutWidget.h"
#include "layout/NoLayout.h"

#include "xml/QtArchGenerator.h"

#define Inherited QListViewItem

LayoutItem::LayoutItem( LayoutItemType		layoutItemType,	
			LayoutItem*		parent,
			const char*		treeText,
			const QPixmap*		treePixmap )
    :
    Inherited( parent, LastChild( parent ), treeText ),
    myLayoutItemType( layoutItemType )
{
    if( treePixmap )
    {
	setPixmap( 0, *treePixmap );
    }

    setOpen( TRUE );
}


LayoutItem::LayoutItem( LayoutItemType		layoutItemType,	
			QListView*		parent,
			const char*		treeText,
			const QPixmap*		treePixmap )
    :
    Inherited( parent, treeText ),
    myLayoutItemType( layoutItemType )
{
    if( treePixmap )
    {
	setPixmap( 0, *treePixmap );
    }

    setOpen( TRUE );
}


LayoutItem::~LayoutItem()
{
}


LayoutItem::LayoutItemType LayoutItem::GetParentType( void ) const
{
    LayoutItemType rc = LayoutItem::LI_None;

    // Need hack to make this non const due do ktreelist bug.  FixMe.
    LayoutItem* parentItem = static_cast<LayoutItem*>(parent());
    if( parentItem )
    {
	rc = parentItem->GetType();
    }
    
    return rc;
}


QWidget* LayoutItem::GetParentWidget( bool	/*theFirstLevel*/ ) const
{
    QWidget* parentWidget = NULL;

    // If not at root
    LayoutItem* parentItem = static_cast<LayoutItem*>(parent());
    if( parentItem )
    {
	parentWidget = parentItem->GetParentWidget( FALSE );
    }
    
    return parentWidget;
}


QLayout* LayoutItem::RecreateLayout( QWidget*	/*theBaseWindow*/ )
{
    return NULL;
}


void LayoutItem::AddToLayout( QBoxLayout*	/*theBoxLayout*/ )
{
}


void LayoutItem::AddToLayout( QGridLayout*	/*theGridLayout*/,
			      int		/*theRowNum*/,
			      int		/*theColNum*/ )
{
}


LayoutItem* LayoutItem::CreateLayoutItem( 
    LayoutItemType	theLayoutItemType,	
    LayoutItem*		theParent
    )
{
    LayoutItem* item = NULL;
    
    switch( theLayoutItemType )
    {
	case LI_NoLayout:
	{
	    item = new NoLayout( theParent );
	    break;
	}
	case LI_BoxLayout:
	{
	    item = new BoxLayout( theParent );
	    break;
	}
	case LI_BoxSpacing:
	{
	    item = new BoxSpacing( theParent );
	    break;
	}
	case LI_BoxStretch:
	{
	    item = new BoxStretch( theParent );
	    break;
	}
	case LI_GridLayout:
	{
	    item = new GridLayout( theParent );
	    break;
	}
	case LI_GridRow:
	{
	    item = new GridRow( theParent );
	    break;
	}
	case LI_GridSpacer:
	{
	    item = new GridSpacer( theParent );
	    break;
	}
	case LI_LayoutWidget:
	{
	    item = new LayoutWidget( theParent );
	    break;
	}
	case LI_None:
	default:
	{
	    break;
	}
    }

    return item;
}


LayoutItem* LayoutItem::CreateLayoutItem( 
    LayoutItemType	theLayoutItemType,	
    QListView*		theParent
    )
{
    LayoutItem* item = NULL;
    
    switch( theLayoutItemType )
    {
	case LI_NoLayout:
	{
	    item = new NoLayout( theParent );
	    break;
	}
	case LI_BoxLayout:
	{
	    item = new BoxLayout( theParent );
	    break;
	}
	case LI_BoxSpacing:
	{
	    item = new BoxSpacing( theParent );
	    break;
	}
	case LI_BoxStretch:
	{
	    item = new BoxStretch( theParent );
	    break;
	}
	case LI_GridLayout:
	{
	    item = new GridLayout( theParent );
	    break;
	}
	case LI_GridRow:
	{
	    item = new GridRow( theParent );
	    break;
	}
	case LI_GridSpacer:
	{
	    item = new GridSpacer( theParent );
	    break;
	}
	case LI_LayoutWidget:
	{
	    item = new LayoutWidget( theParent );
	    break;
	}
	case LI_None:
	default:
	{
	    break;
	}
    }

    return item;
}


LayoutItem* LayoutItem::InsertNewLayoutItem( 
    LayoutItemType	theItem,
    const QString&	theId 
    )
{
    LayoutItem* newItem = 0;

    int pos = theId.find( '.' );
    if( pos == -1 )
    {
	// The new item needs to be added to this layout.
	newItem = CreateLayoutItem( theItem, this );
    }
    else
    {
	// The item needs to be added to a sublayout.
	QString index = theId.left( pos );
	QString newId = theId.mid( pos + 1, theId.length() - pos - 1 );

	// The child index is 0 based, but the index is 1 based.
	int childIndex = index.toInt() - 1;
	LayoutItem* item = 0;
	for( item = static_cast<LayoutItem*>(firstChild());
	     childIndex > 0;
	     item = static_cast<LayoutItem*>(item->nextSibling()),
		 --childIndex 
	    );
	
	if( item )
	{
	    newItem = item->InsertNewLayoutItem( theItem, newId );
	}
    }

    return newItem;
}


void LayoutItem::GenerateSource( QTextStream&	/*theStream*/,
				 const QString&	/*theLayoutId*/,
				 const QString&	/*theLayoutParentWidget*/ ) const
{
}


void LayoutItem::GenerateSourceInBox( QTextStream&	/*theStream*/,
				      const QString&	/*theLayoutVarName*/,
				      const QString&	/*theLayoutId*/,
				      const QString&	/*theLayoutParentWidget*/ ) const
{
}


void LayoutItem::GenerateSourceInGrid( QTextStream&	/*theStream*/,
				       const QString&	/*theLayoutVarName*/,
				       const QString&	/*theLayoutId*/,
				       unsigned int	/*theRowNum*/,
				       unsigned int	/*theColNum*/,
				       const QString&	/*theLayoutParentWidget*/ ) const
{
}


void LayoutItem::DeleteWidget( DlgWidget*	theDlgWidget )
{
    LayoutItem* item = 0;
    for( item = static_cast<LayoutItem*>(firstChild());
	 item;
	 item = static_cast<LayoutItem*>(item->nextSibling()) )
    {
	item->DeleteWidget( theDlgWidget );
    }
}


void LayoutItem::WidgetChanged( 
    const QString& oldName, 
    const DlgWidget* widget 
    )
{
    LayoutItem* item = 0;
    for( item = static_cast<LayoutItem*>(firstChild()); 
         item;
         item = static_cast<LayoutItem*>(item->nextSibling()) )
    {
        item->WidgetChanged( oldName, widget );
    }
}


const char* LayoutItem::Type2String() const
{
    switch( myLayoutItemType )
    {
	case LI_NoLayout:
	{
	    return "No_Layout";
	    break;
	}
	case LI_BoxLayout:
	{
	    return "Box_Layout";
	    break;
	}
	case LI_BoxSpacing:
	{
	    return "Box_Spacing";
	    break;
	}
	case LI_BoxStretch:
	{
	    return "Box_Stretch";
	    break;
	}
	case LI_GridLayout:
	{
	    return "Grid_Layout";
	    break;
	}
	case LI_GridRow:
	{
	    return "Grid_Row";
	    break;
	}
	case LI_GridSpacer:
	{
	    return "Grid_Spacer";
	    break;
	}
	case LI_LayoutWidget:
	{
	    return "Layout_Widget";
	    break;
	}
	case LI_None:
	default:
	{
	    return "";
	    break;
	}
    }
}


void LayoutItem::setup()
{
    Inherited::setup();

    const QPixmap* itemPixmap = pixmap( 0 );
    if( itemPixmap )
    {
	int pixmapHeight = itemPixmap->height() + 2 * listView()->itemMargin();
	setHeight( QMAX( height(), pixmapHeight ) );
    }
}


void LayoutItem::insertItem(
    QListViewItem*	theItem
    )
{
    Inherited::insertItem( theItem );

    if( theItem )
    {
	theItem->invalidateHeight();
    }
}


LayoutItem::LayoutItemType LayoutItem::String2Type( const QString& theString )
{
    LayoutItemType type = LI_None;
    
    if( theString == "No_Layout" )
    {
	type = LI_NoLayout;
    }
    else if( theString == "Box_Layout" )
    {
	type = LI_BoxLayout;
    }
    else if( theString == "Box_Spacing" )
    {
	type = LI_BoxSpacing;
    }
    else if( theString == "Box_Stretch" )
    {
	type = LI_BoxStretch;
    }
    else if( theString == "Grid_Layout" )
    {
	type = LI_GridLayout;
    }
    else if( theString == "Grid_Row" )
    {
	type = LI_GridRow;
    }
    else if( theString == "Grid_Spacer" )
    {
	type = LI_GridSpacer;
    }
    else if( theString == "Layout_Widget" )
    {
	type = LI_LayoutWidget;
    }

    return type;
}


void LayoutItem::SaveChildren( QtArchGenerator&	generator ) const
{
  LayoutItem* item = 0;

  generator.WriteStartList( "Children" );
  for( item = static_cast<LayoutItem*>(firstChild()); 
       item; 
       item = static_cast<LayoutItem*>(item->nextSibling()) )
  {
    item->SaveContents( generator );
  }
  generator.WriteEndBlock();
}

void LayoutItem::RestoreChildren( const XMLValue &value, LayoutHelper *helper )
{
  // value should be a list
  if( value.Type() != XMLValue::XMLList )
    return;

  XMLValueList list = value.AsList();
  QListIterator<XMLValue> it( list );
  for( ; it.current(); ++it )
  {
    LayoutItemType type = String2Type( it.current()->GetTypeString() );
    LayoutItem *item = CreateLayoutItem( type, this );
    item->Restore( *(it.current()), helper );
  }
}
    
QListViewItem* LayoutItem::LastChild(
    QListViewItem*	theParent
    )
{
    QListViewItem* lastChild = 0;
 
    if( theParent )
    {
	for( lastChild = theParent->firstChild();
	     lastChild && (lastChild->nextSibling() != 0);
	     lastChild = lastChild->nextSibling() );
    }
   
    return lastChild;
}


