/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: LayoutItem.h,v 2.5 2000/07/23 20:36:43 jeffharris Exp $

*******************************************************************************/


#ifndef LayoutItem_defined
#define LayoutItem_defined

#include <qlayout.h>
#include <qlistview.h>
#include <qpixmap.h>
#include <qstring.h>
#include <qtstream.h>

#include "bmp/PixmapFactory.h"

#include "layout/LayoutItemPropPage.h"

class WidgetLayout;
class DlgWidget;
class QtArchGenerator;
class XMLValue;
class LayoutHelper;

class LayoutItem : public QListViewItem
{
public: // Types

    enum LayoutItemType
    {
        LI_NoLayout,
        LI_BoxLayout,
        LI_BoxSpacing,
        LI_BoxStretch,
        LI_GridLayout,
        LI_GridRow,
        LI_GridSpacer,
        LI_LayoutWidget,
        LI_None = -1
    };

public: // Static methods

    static LayoutItem* CreateLayoutItem(
        LayoutItemType	theLayoutItemType,
        LayoutItem*	theParent
        );
    // Create a new layout item of the given type with the given parent


    static LayoutItem* CreateLayoutItem(
        LayoutItemType	theLayoutItemType,	
        QListView*	theParent
        );
    // Create a new layout item of the given type with the given parent


    static LayoutItemType String2Type( const QString&	theString );
    // Convert a string to a layout item type.


public: // Methods
    
    LayoutItem( LayoutItemType		layoutItemType,
                LayoutItem*		parent,
                const char*		treeLabel,
                const QPixmap*		treePixmap = NULL );
    // Constructor

    LayoutItem( LayoutItemType		layoutItemType,
                QListView*		parent,
                const char*		treeLabel,
                const QPixmap*		treePixmap = NULL );
    // Constructor

    virtual ~LayoutItem() = 0;
    // Destructor

    virtual LayoutItemPropPage* GetPropertyPage( QWidget*	parent,
                                                 bool		topLevel,
                                                 const WidgetLayout&	theWidgetLayout ) = 0;
    // Get the property page for the item.  All the information on the page 
    // should be updated.  The caller is responsible for freeing the
    // page when finished.

    LayoutItemType GetType( void ) const;
    // Get the type of the layout

    LayoutItemType GetParentType( void ) const;
    // Get the type of the item's parent

    virtual QWidget* GetParentWidget( bool	theFirstLevel = TRUE ) const;
    // Get the parent widget for the item on the dialog window.  If NULL is
    // returned, the parent is the widget's original parent.

    //
    // Layout creation
    //

    virtual QLayout* RecreateLayout( QWidget*	theBaseWindow );
    // Create a new layout for theBaseWindow and add all of its children to it.

    virtual void AddToLayout( QBoxLayout*	theBoxLayout );
    // Add the item and all its children to a box layout.

    virtual void AddToLayout( QGridLayout*	theGridLayout,
                              int		theRowNum,
                              int		theColNum );
    // Add the item and all its children to a grid layout.


    //
    // Loading and Saving
    //

    virtual void SaveContents( QtArchGenerator & ) const = 0;
    // Save the contents of this layout item to theStream with theId as
    // its position identifier.

    virtual void Restore( const XMLValue &, LayoutHelper * ) = 0;
    // Restore the item from the given XMLValue
    
    virtual LayoutItem* InsertNewLayoutItem( LayoutItemType	theItem,
                                             const QString&	theId );
    // Insert theItem into its place in the tree, as specified by theId.

    virtual bool RestoreKeyValue( const QString&	theKey,
                                  const QString&	theValue,
                                  const DlgWidget*	theDlgWidget ) = 0;
    // Restore a key/value pair for the item.  theDlgWidget is the dialog
    // widget associated with theValue, if any.

    //
    // Code generation
    //

    virtual void GenerateSource( QTextStream&	theStream,
                                 const QString&	theLayoutId = "1",
                                 const QString&	theLayoutParentWidget = QString() ) const;
    // Generate the source for the entire layout tree.

    virtual void GenerateSourceInBox( QTextStream&	theStream,
                                      const QString&	theLayoutVarName,
                                      const QString&	theLayoutId,
                                      const QString&	theLayoutParentWidget ) const;
    // Generate the source code in a box layout

    virtual void GenerateSourceInGrid( QTextStream&	theStream,
                                       const QString&	theLayoutVarName,
                                       const QString&	theLayoutId,
                                       unsigned int	theRowNum,
                                       unsigned int	theColNum,
                                       const QString&	theLayoutParentWidget ) const;
    // Generate the source code in a grid layout


    //
    // Misc
    //

    virtual void DeleteWidget( DlgWidget*	theDlgWidget );
    // Delete the specified widget from the layout

    virtual void WidgetChanged( const QString& oldName, 
                                const DlgWidget* widget );
    // Notification that the given widget has changed

    const char* Type2String() const;
    // Convert this item's type to a string.


    //
    // Overridden methods
    //

    virtual void setup();

    virtual void insertItem(
        QListViewItem*	theItem
        );
    

protected: // Methods

    //
    // Loading and Saving
    //

    void SaveChildren( QtArchGenerator & ) const;
    // Save the item's children

    void RestoreChildren( const XMLValue &, LayoutHelper * );
    // Restore the item's children (likely to be recursive...)

private: // Static Methods

    static QListViewItem* LastChild(
        QListViewItem*	theParent
        );

private: // Attributes

    LayoutItemType	myLayoutItemType;
    // Type of this layout
};


inline LayoutItem::LayoutItemType LayoutItem::GetType( void ) const
{
    return myLayoutItemType;
}




#endif // LayoutItem_defined

