/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: LayoutWidget.cpp,v 2.13 2000/07/23 20:36:43 jeffharris Exp $

*******************************************************************************/


#include "layout/LayoutWidget.h"
#include "layout/LayoutWidgetPropPage.h"
#include "layout/LayoutHelper.h"

#include "widget/DlgWidget.h"

#include "xml/QtArchGenerator.h"

#define Inherited LayoutItem

LayoutWidget::LayoutWidget(
    LayoutItem*	theParent
    )
    :
    Inherited( LayoutItem::LI_LayoutWidget,
               theParent,
               GetTreeLabel(), 
               &PixmapFactory::GetPixmap( Frame_bmp ) ),
    myDlgWidget( NULL ),
    myBoxStretch( 1 ),
    myAlignment( 0 ),
    myLayout( NULL )
{
}


LayoutWidget::LayoutWidget(
    QListView*	theParent
    )
    :
    Inherited( LayoutItem::LI_LayoutWidget, 
               theParent,
               GetTreeLabel(), 
               &PixmapFactory::GetPixmap( Frame_bmp ) ),
    myDlgWidget( NULL ),
    myBoxStretch( 1 ),
    myAlignment( 0 ),
    myLayout( NULL )
{
}


LayoutWidget::~LayoutWidget()
{
    //delete myLayout;

    // Remove the widget from any layout
    SetDlgWidget( NULL, TRUE /*isInDtor*/ );
}


LayoutItemPropPage* LayoutWidget::GetPropertyPage( 
    QWidget*	parent,
    bool		/*topLevel*/,
    const WidgetLayout&	theWidgetLayout )
{
    return new LayoutWidgetPropPage( this, 
                                     theWidgetLayout, 
                                     parent );
}


QWidget* LayoutWidget::GetParentWidget( bool	theFirstLevel ) const
{
    QWidget* parentWidget = NULL;

    // At the first call, you have to go up to the parent to find the widget's
    // parent widget.  All calls after the first can return the dlgwidget's
    // widget, if it has one.
    if( !theFirstLevel && myDlgWidget )
    {
        parentWidget = myDlgWidget->GetWidget();
    }
    else
    {
        parentWidget = LayoutItem::GetParentWidget( FALSE );
    }

    return parentWidget;
}


QLayout* LayoutWidget::RecreateLayout( 
    QWidget*	/*theBaseWindow */
    )
{
    QLayout* layout = 0;

    delete myLayout;
    myLayout = 0;

    if( WidgetHasOwnLayout() )
    {
        LayoutWidget* item = 0;
        for( item = static_cast<LayoutWidget*>(firstChild());
             item;
             item = static_cast<LayoutWidget*>(item->nextSibling()) )
        {   
            const DlgWidget* widget = item->GetDlgWidget();
            item->SetDlgWidget( 0 );
            item->SetDlgWidget( widget );

            item->RecreateLayout( myDlgWidget->GetWidget() );
        }
    }
    else
    {
        // If the widget has a child layout, recreate it
        if( firstChild() )
        {       
            LayoutItem* item = static_cast<LayoutItem*> (firstChild());
            
            if( myDlgWidget )
            {
                layout = item->RecreateLayout( myDlgWidget->GetWidget() );
            }
        }
    }

    myLayout = layout;
    return layout;
}


void LayoutWidget::AddToLayout( QBoxLayout*	theBoxLayout )
{
    if( myDlgWidget )
    {
        RecreateLayout( 0 );

        theBoxLayout->addWidget( myDlgWidget->GetWidget(),
                                 myBoxStretch,
                                 myAlignment );
    }
}


void LayoutWidget::AddToLayout( QGridLayout*	theGridLayout,
                                int		theRowNum,
                                int		theColNum )
{
    if( myDlgWidget )
    {
        RecreateLayout( 0 );

        theGridLayout->addWidget( myDlgWidget->GetWidget(),
                                  theRowNum,
                                  theColNum,
                                  myAlignment );
    }
}


void LayoutWidget::SaveContents( QtArchGenerator &generator ) const
{
  generator.WriteStartBlock( Type2String() );

  if( myDlgWidget )
  {
    generator.WriteSingleLine( "Widget", myDlgWidget->GetWidgetName() );
  }

  if( !IsLayoutWidgetChild() )
  {
    generator.WriteXMLValue( "BoxStretch", myBoxStretch );
    generator.WriteXMLValue( "Alignment", myAlignment );
  }
    
  SaveChildren( generator );

  generator.WriteEndBlock();
}


bool LayoutWidget::RestoreKeyValue( const QString&	theKey,
                                    const QString&	theValue,
                                    const DlgWidget*	theDlgWidget )
{
    bool rc = TRUE;

    if( theKey == "Widget" )
    {
        if( theDlgWidget )
        {
            SetDlgWidget( theDlgWidget );
        }
        else
        {
            // If we didn't have a widget, the key wouldn't have been saved.
            rc = FALSE;
        }
    }
    else if( theKey == "BoxStretch" )
    {
        myBoxStretch = theValue.toInt();
    }
    else if( theKey == "Alignment" )
    {
        myAlignment = theValue.toInt();
    }
    else
    {
        rc = FALSE;
    }
    
    return rc;
}

void LayoutWidget::Restore( const XMLValue &value, LayoutHelper *helper )
{
  // Sanity check
  if( value.GetTypeString() != Type2String() )
    return;

  const XMLValueMap map = value.AsNested();

  const DlgWidget *widget = helper->GetWidget( map["Widget"].AsQString() );
  if( widget )
  {
    SetDlgWidget( widget );
  }
  myBoxStretch = map["BoxStretch"].AsInt();
  myAlignment = map["Alignment"].AsInt();

  RestoreChildren( map["Children"], helper );
}


void LayoutWidget::GenerateSourceInBox( QTextStream&	theStream,
                                        const QString&	theLayoutVarName,
                                        const QString&	theLayoutId,
                                        const QString&	theLayoutParentWidget ) const
{
    if( myDlgWidget )
    {
        theStream << '\t' << theLayoutVarName << "->addWidget( " <<
            myDlgWidget->GetGeneratedVarName() << ", " <<
            myBoxStretch << ", " << myAlignment << " );\n";
	
        GenerateSource( theStream, theLayoutId, theLayoutParentWidget );
    }
}


void LayoutWidget::GenerateSourceInGrid( QTextStream&	theStream,
                                         const QString&	theLayoutVarName,
                                         const QString&	theLayoutId,
                                         unsigned int	theRowNum,
                                         unsigned int	theColNum,
                                         const QString&	theLayoutParentWidget ) const
{
    if( myDlgWidget )
    {
        theStream << '\t' << theLayoutVarName << "->addWidget( " <<
            myDlgWidget->GetGeneratedVarName() << ", " <<
            theRowNum << ", " << theColNum << ", " << myAlignment << " );\n";
	
        GenerateSource( theStream, theLayoutId, theLayoutParentWidget );
        
    }
}


void LayoutWidget::GenerateSource( 
    QTextStream&	theStream,
    const QString&	theLayoutId,
    const QString&	/*theLayoutParentWidget*/
    ) const
{
    QString varName = myDlgWidget ? 
        myDlgWidget->GetGeneratedVarName() : QString();

    LayoutItem* item = 0;
    int pos = 1;
    QString idStr;
    for( item = static_cast<LayoutItem*>(firstChild());
         item;
         item = static_cast<LayoutItem*>(item->nextSibling()), ++pos )
    {
        idStr = theLayoutId + '_' + QString::number( pos );

        item->GenerateSource( theStream, idStr, varName );
    }    
}


void LayoutWidget::DeleteWidget( DlgWidget*	theDlgWidget )
{
    if( theDlgWidget == myDlgWidget )
    {
        SetDlgWidget( NULL );

        // Delete any child layout it may have
        if( childCount() > 0 )
        {
            delete myLayout;
            myLayout = NULL;
	    
            delete firstChild();
        }
    }
    
    // Still loop in case the widget is used more than once.
    LayoutItem::DeleteWidget( theDlgWidget );
}


void LayoutWidget::WidgetChanged( 
    const QString& oldName, 
    const DlgWidget* widget 
    )
{
    if( text( 0 ) == oldName )
    {
        SetDlgWidget( 0 );
        SetDlgWidget( widget );
    }

    Inherited::WidgetChanged( oldName, widget );
}


bool LayoutWidget::WidgetHasOwnLayout() const
{
    bool haveChildren = FALSE;
    
    if( myDlgWidget )
    {
        switch( myDlgWidget->GetType() )
        {
            case DlgWidget::DW_HBox:
            case DlgWidget::DW_VBox:
            case DlgWidget::DW_Grid:
            case DlgWidget::DW_Splitter:
            case DlgWidget::DW_HButtonGroup:
            case DlgWidget::DW_VButtonGroup:
            case DlgWidget::DW_HGroupBox:
            case DlgWidget::DW_VGroupBox:
            {
                haveChildren = TRUE;
                break;
            }
            default:
            {
                break;
            }
        }
    }
    
    return haveChildren;
}


bool LayoutWidget::IsLayoutWidgetChild() const
{
    bool isLayoutWidgetChild = FALSE;

    LayoutItem* parentItem = static_cast<LayoutItem*>(parent());
    if( parentItem )
    {
        isLayoutWidgetChild = 
            (parentItem->GetType() == LayoutItem::LI_LayoutWidget);
    }
    
    return isLayoutWidgetChild;
}


void LayoutWidget::GenerateContents( QTextStream&	theStream,
                                     const QString&	/*theVarName*/,
                                     const QString&	theLayoutId,
                                     const QString&	/*theLayoutParentWidget*/ ) const
{
/*
  // If the layout parent widget is given, then this widget needs to 
  // recreate itself in the parent widget
  if( !theLayoutParentWidget.isEmpty() )
  {
  theStream << '\t' << myDlgWidget->GetGeneratedVarName() <<
  "->recreate( " << theLayoutParentWidget << 
  ", 0, QPoint(0,0), TRUE );\n";
  }
*/

    // If the widget has a child layout, generate it
    LayoutItem* child = static_cast<LayoutItem*>(firstChild());
    if( child )
    {
        // If there is a sub-layout, then this widget becomes the parent
        // of all the items in the sub-layout.
        child->GenerateSource( theStream, 
                               theLayoutId, 
                               myDlgWidget->GetGeneratedVarName() );
    }
}


void LayoutWidget::SetDlgWidget( 
    const DlgWidget*	theDlgWidget,
    bool		isInDtor
    )
{
    // Only set if they're different
    if( theDlgWidget != myDlgWidget )
    {
        if( myDlgWidget )
        {
            const_cast<DlgWidget*>(myDlgWidget)->RemoveFromLayout();
        }
	
        myDlgWidget = theDlgWidget;

        if( !isInDtor )
        {
            QString label = GetTreeLabel();
            QPixmap pixmap = PixmapFactory::GetPixmap( Frame_bmp );

            if( myDlgWidget )
            {
                const_cast<DlgWidget*>(myDlgWidget)->SetInLayout( 
                    GetParentWidget() );

                label = myDlgWidget->GetWidgetName();
                pixmap = myDlgWidget->GetPixmap();
            }

            setText( 0, label );
            setPixmap( 0, pixmap );

            listView()->repaint();
        }
    }
}


QString LayoutWidget::GetTreeLabel()
{
    static QString treeLabel = QObject::tr( "Layout Widget" );
    return treeLabel;
}
