/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: LayoutWidget.h,v 2.7 2000/07/23 20:36:43 jeffharris Exp $

*******************************************************************************/


#ifndef LayoutWidget_included
#define LayoutWidget_included

#include <qlayout.h>

#include "layout/LayoutItem.h"

class DlgWidget;
class QListView;
class QtArchGenerator;

class LayoutWidget : public LayoutItem
{
public: // Methods

    LayoutWidget(
	LayoutItem*	theParent
	);
    // Constructor

    LayoutWidget(
	QListView*	theParent
	);
    // Constructor

    virtual ~LayoutWidget();
    // Destructor

    virtual LayoutItemPropPage* GetPropertyPage( QWidget*	parent,
						 bool		topLevel,
						 const WidgetLayout&	theWidgetLayout );
    // Get the property page for the item.  All the information on the page 
    // should be updated.  The caller is responsible for freeing the
    // page when finished.

    virtual QWidget* GetParentWidget( bool	theFirstLevel = TRUE ) const;
    // Get the parent widget for the item on the dialog window.  If NULL is
    // returned, the parent is the widget's original parent.

    //
    // Layout creation
    //

    virtual QLayout* RecreateLayout( QWidget*	theBaseWindow );
    // Create a new layout for theBaseWindow and add all of its children to it.

    virtual void AddToLayout( QBoxLayout*	theBoxLayout );
    // Add the item and all its children to a box layout.

    virtual void AddToLayout( QGridLayout*	theGridLayout,
			      int		theRowNum,
			      int		theColNum );
    // Add the item and all its children to a grid layout.



    //
    // Loading and Saving
    //

    virtual void SaveContents( QtArchGenerator & ) const;
    // Save the contents of this layout item to theStream with theId as
    // its position identifier.
    
    virtual void Restore( const XMLValue &, LayoutHelper * );
    // Restore the item from the given XMLValue
    
    virtual bool RestoreKeyValue( const QString&	theKey,
				  const QString&	theValue,
				  const DlgWidget*	theDlgWidget );
    // Restore a key/value pair for the item.  theDlgWidget is the dialog
    // widget associated with theValue, if any.


    //
    // Code generation
    //

    virtual void GenerateSource( 
        QTextStream&	theStream,
        const QString&	theLayoutId = "1",
        const QString&	theLayoutParentWidget = QString() 
        ) const;
    // Generate the source for the entire layout tree.

    virtual void GenerateSourceInBox( QTextStream&	theStream,
				      const QString&	theLayoutVarName,
				      const QString&	theLayoutId,
				      const QString&	theLayoutParentWidget ) const;
    // Generate the source code in a box layout.


    virtual void GenerateSourceInGrid( QTextStream&	theStream,
				       const QString&	theLayoutVarName,
				       const QString&	theLayoutId,
				       unsigned int	theRowNum,
				       unsigned int	theColNum,
				       const QString&	theLayoutParentWidget ) const;
    // Generate the source code in a grid layout.


    //
    // Misc
    //

    virtual void DeleteWidget( DlgWidget*	theDlgWidget );
    // Delete the specified widget from the layout

    virtual void WidgetChanged( const QString& oldName, 
                                const DlgWidget* widget );
    // Notification that the given widget has changed

    bool WidgetHasOwnLayout() const;
    // Does the widget have its own layout management of its children

    bool IsLayoutWidgetChild() const;
    // Is the layout widget a child of another layout widget
    
    //
    // Get/Set Properties
    //

    // Dialog widget
    const DlgWidget* GetDlgWidget( void ) const;
    void SetDlgWidget( const DlgWidget*	theDlgWidget,
		       bool		isInDtor = FALSE );

    // Box stretch
    int GetBoxStretch( void ) const;
    void SetBoxStretch( int	theStretch );

    // Alignment
    int GetAlignment( void ) const;
    void SetAlignment( int	theAlignment );

private: // Methods

    void GenerateContents( QTextStream&		theStream,
			   const QString&	theVarName,
			   const QString&	theLayoutId,
			   const QString&	theLayoutParentWidget ) const;
    // Generate the contents of the box layout

private: // Class Methods

    static QString GetTreeLabel();
    // Get the label for the item in the tree
    
private: // Attributes

    //
    // Properties
    //

    const DlgWidget*	myDlgWidget;
    int			myBoxStretch;
    int			myAlignment;

    QLayout*		myLayout;
    // The layout thay may be defined for this widget.
};


inline const DlgWidget* LayoutWidget::GetDlgWidget( void ) const
{
    return myDlgWidget;
}


inline int LayoutWidget::GetBoxStretch( void ) const
{
    return myBoxStretch;
}


inline void LayoutWidget::SetBoxStretch( int	theStretch )
{
    myBoxStretch = theStretch;
}


inline int LayoutWidget::GetAlignment( void ) const
{
    return myAlignment;
}


inline void LayoutWidget::SetAlignment( int	theAlignment )
{
    myAlignment = theAlignment;
}

#endif // LayoutWidget_included
