/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: WidgetPropDialog.cpp,v 2.5 2000/07/23 20:37:45 jeffharris Exp $

*******************************************************************************/


#include <qstring.h>
#include <qlist.h>
#include <qstrlist.h>

#include "WidgetPropDialog.h"

#include "prop/WidgetVarMapPage.h"
#include "prop/WidgetCommonPage.h"
#include "prop/WidgetColorPage.h"
#include "prop/WidgetGeometryPage.h"

#define Inherited WidgetPropDialogData

WidgetPropDialog::WidgetPropDialog
(
    DlgWidget* widget,
    QWidget* parent,
    const char* name
)
    :
    Inherited( parent, name ),
    itsDlgWidget( widget )
{
  QString str = tr( "Widget Properties - %1" ).arg( widget->GetWidgetName() );
  setCaption( str );

  itsVarMapPage = new WidgetVarMapPage( widget, this, "Mapping" );
  itsGeometryPage = new WidgetGeometryPage( widget, this, "Geometry" );
  itsCommonPage = new WidgetCommonPage( widget, this, "Widget" );
  itsColorPage = new WidgetColorPage( widget, this, "Color" );    

  addTab( itsVarMapPage, tr( "Mapping" ) );
  addTab( itsGeometryPage, tr( "Geometry" ) );
  addTab( itsCommonPage, tr( "Widget" ) );
  addTab( itsColorPage, tr( "Colors & Font" ) );

  CreateWidgetPages();
  
  setCancelButton();
  setApplyButton();

  connect( this, SIGNAL(applyButtonPressed()), SLOT(Apply()) );

  resize( sizeHint() );
}


WidgetPropDialog::~WidgetPropDialog()
{
}


void WidgetPropDialog::ChangeWidget( DlgWidget *newWidget )
{
  debug( "ChangeWidget called" );

  DlgWidget::Type oldType = itsDlgWidget->GetType();
  
  itsDlgWidget = newWidget;

  QString str =
    tr( "Widget Properties - %1" ).arg( newWidget->GetWidgetName() );
  setCaption( str );

  // Change common pages
  itsVarMapPage->NewWidget( newWidget );
  itsGeometryPage->NewWidget( newWidget );
  itsCommonPage->NewWidget( newWidget );
  itsColorPage->NewWidget( newWidget );

  // Check if the widget pages should change, too
  if( oldType != newWidget->GetType() )
  {
    // Ensure that no widget specific page is shown
    showPage( itsVarMapPage );
    
    // Remove all type specific pages
    QWidget *page;
    for( page = pageList.first(); page; page = pageList.next() )
    {
      removePage( page );
      disconnect( page );
    }
    
    // Insert the pages for the new widget
    CreateWidgetPages();
  } else
  {
    // Refresh the widget pages
    emit NewWidget( newWidget );
  }
}

void WidgetPropDialog::accept()
{
  Inherited::accept();
  Apply();
}

void WidgetPropDialog::reject()
{
  Inherited::reject();
}

void WidgetPropDialog::Apply()
{
    itsVarMapPage->Apply( itsDlgWidget );
    itsGeometryPage->Apply( itsDlgWidget );
    itsCommonPage->Apply( itsDlgWidget );
    itsColorPage->Apply( itsDlgWidget );

    emit Apply( itsDlgWidget );

    itsDlgWidget->GetWidget()->repaint();
    itsDlgWidget->Changed();
}

void WidgetPropDialog::CreateWidgetPages()
{
  QList<QWidget> pages;
  QStrList labels;
    
  itsDlgWidget->GetPropertyPages( this, pages, labels );

  QListIterator<QWidget> pageIter( pages );
  QStrListIterator labelIter( labels );
  QWidget* page;
  const char* label;
    
  while( (page = pageIter()) )
  {
    label = labelIter();
	
    addTab( page, label );

    page->connect( this, SIGNAL(Apply(DlgWidget*)), 
                   SLOT(Apply(DlgWidget*)) );
    page->connect( this, SIGNAL(NewWidget(DlgWidget*)), 
                   SLOT(NewWidget(DlgWidget*)) );
    pageList.prepend( page );
  }

}
