/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgLabel.cpp,v 2.13 2000/07/23 20:38:01 jeffharris Exp $

*******************************************************************************/


#include "widget/DlgLabel.h"

#include "prop/LabelPage.h"

#include "GuiTools.h"

#include "xml/QtArchExpat.h"
#include "xml/QtArchGenerator.h"

#define Inherited DlgFrame

QStrList DlgLabel::itsSignals;
int DlgLabel::itsNextWidgetNum = 1;

DlgLabel::DlgLabel() :
    Inherited( DlgWidget::DW_Label )
{
}


DlgLabel::DlgLabel(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgLabel::~DlgLabel()
{
}


void DlgLabel::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new LabelPage( this, parent, "Label" ) );

    labels.append( tr( "Label" ) );
}

void DlgLabel::Save( QtArchGenerator &generator )
{
  generator.WriteStartBlock( "Label" );
  
  QLabel* widget = GetLabel();

  generator.WriteSingleLine( "Text", widget->text() );

  generator.WriteSingleLine( "Alignment",
                             AlignmentToStr( widget->alignment() ));

  generator.WriteXMLValue( "Indent", widget->indent() );

  generator.WriteXMLValue( "TextFormat", 
                           TextFormatToStr( widget->textFormat() ) );

  Inherited::Save( generator );

  generator.WriteEndBlock();

}


bool DlgLabel::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    QLabel* widget = GetLabel();

    // Compare to known keys
    if( key == "Text" )
    {
	widget->setText( value );
    } 
    else if( key == "Alignment" )
    {
        bool ok = FALSE;
        int alignment = value.toInt( &ok );
        if( !ok )
        {
            alignment = StrToAlignment( value );
        }
        
	widget->setAlignment( alignment );
    }
    else if( key == "AutoResize" )
    {
        // Deprecated
    }
    else if( key == "Margin" )
    {
	widget->setIndent( value.toInt() );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}

void DlgLabel::RestoreXML( const XMLValue &value )
{
    // Sanity check
    if( value.GetTypeString() != "Label" )
        return;

    const XMLValueMap map = value.AsNested();

    // Create if necessary
    if( ! GetLabel() )
    {
        CreateQtWidget( 0 );
    }
  
    QLabel* widget = GetLabel();

    // Must set the text format before setting the text
    widget->setTextFormat( StrToTextFormat( map["TextFormat"].AsQString() ) );

    widget->setText( map["Text"].AsQString() );

    int alignment = StrToAlignment( map["Alignment"].AsQString() );
    widget->setAlignment( alignment );

    widget->setIndent( map["Indent"].AsInt() );

    // We inherit from Frame
    Inherited::RestoreXML( map["Frame"] );

}


void DlgLabel::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QLabel* widget = GetLabel();

    // Must set the text format before setting the text
    stream << varName << "setTextFormat( Qt::"
           << TextFormatToStr( widget->textFormat() ) << " );" << endl;

    stream << varName << "setText( ";
    GenerateText( widget->text(), widgetLayout, stream );
    stream << " );" << endl;
    
    stream << varName << "setAlignment( " 
           << AlignmentToStr( widget->alignment() ) << " );" << endl;
    
    stream << varName << "setIndent( " << widget->indent() << " );" << endl;
}


void DlgLabel::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgLabel::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
    }

    return itsSignals;
}


void DlgLabel::CreateQtWidget(
    const char*	theName
    )
{
  QLabel* widget = 
    new QLabel( GetParent(),
                CreateName( "Label", 
                            theName, 
                            itsNextWidgetNum ) );

  SetQtWidget( widget );
}
