/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgListView.cpp,v 2.8 2000/07/23 20:38:01 jeffharris Exp $

*******************************************************************************/


#include "widget/DlgListView.h"

#include "prop/ListViewColumnPage.h"
#include "prop/ListViewPage.h"

#include "GuiTools.h"

#include "xml/QtArchExpat.h"
#include "xml/QtArchGenerator.h"

#define Inherited DlgScrollView

QStrList DlgListView::itsSignals;
int DlgListView::itsNextWidgetNum = 1;

DlgListView::DlgListView() :
    Inherited( DlgWidget::DW_ListView ),
    itsNumHeaders( 0 ),
    isFirstHeaderLoaded( FALSE )
{
}


DlgListView::DlgListView(
    DlgWidget::Type	theType
    ) :
    Inherited( theType ),
    itsNumHeaders( 0 ),
    isFirstHeaderLoaded( FALSE )
{
}


DlgListView::~DlgListView()
{
}


void DlgListView::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new ListViewPage( this, parent, "ListView" ) );
    pages.append( new ListViewColumnPage( this,
					  parent,
					  "List View Columns" ) );

    labels.append( tr( "List View" ) );
    labels.append( tr( "List View Columns" ) );
}

void DlgListView::Save( QtArchGenerator &generator )
{
  generator.WriteStartBlock( "ListView" );
  
  QListView* widget = GetListView();

  generator.WriteStartList( "HeaderInformation" );
  for( unsigned int i = 0; i < itsNumHeaders; ++i )
  {
    generator.WriteStartBlock( "Header" );
    generator.WriteXMLValue( "Width", widget->columnWidth( i ) );
    generator.WriteSingleLine( "WidthMode",
                               WidthModeToStr( widget->columnWidthMode( i ) ) );
    generator.WriteXMLValue( "Alignment", widget->columnAlignment( i ));
    generator.WriteSingleLine( "Text", widget->columnText( i ));
    
    generator.WriteEndBlock();
  }
  generator.WriteEndBlock();
  
  generator.WriteXMLValue( "TreeStepSize", widget->treeStepSize() );
  generator.WriteXMLValue( "MultiSelection", widget->isMultiSelection() );
  generator.WriteXMLValue( "AllColumnsShowFocus", widget->allColumnsShowFocus() );
  generator.WriteXMLValue( "ItemMargin", widget->itemMargin() );
  generator.WriteXMLValue( "RootIsDecorated", widget->rootIsDecorated() );

  Inherited::Save( generator );

  generator.WriteEndBlock();
}


bool DlgListView::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
        if( key == "Name" )
        {
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
        }
    }

    QListView* widget = GetListView();

    // Compare to known keys
    if( key == "Header" )
    {
        int columnWidth = 0;
        int index = 0;

        int pos = value.find( ' ' );
        if( pos > -1 )
        {
	    columnWidth = value.mid( 0, pos ).toInt();
        }
        else
        {
	    retVal = FALSE;
        }
	
        QListView::WidthMode mode = QListView::Maximum;

        index = pos + 1;
        pos = value.find( ' ', index );
        if( pos > -1 )
        {
	    mode = StrToWidthMode( value.mid( index, pos - index ) );
        }
        else
        {
	    retVal = FALSE;
        }

        int columnAlignment = 0;
	
        index = pos + 1;
        pos = value.find( ' ', index );
        if( pos > -1 )
        {
	    columnAlignment = value.mid( index, pos - index ).toInt();
        }
        else
        {
	    retVal = FALSE;
        }
	
        QString label = value.mid( pos + 1, value.length() - pos - 1 );
	
        if( retVal )
        {
	    columnWidth = (mode == QListView::Maximum) ? -1 : columnWidth;

	    // The ctor places one column in the widget, over write the
	    // values.
	    if( !isFirstHeaderLoaded && (itsNumHeaders == 1) )
	    {
                widget->setColumnText( 0, label );
                widget->setColumnWidth( 0, columnWidth );
                widget->setColumnWidthMode( 0, mode );
                widget->setColumnAlignment( 0, columnAlignment );

                isFirstHeaderLoaded = TRUE;
	    }
	    else
	    {
                int index = widget->addColumn( label, columnWidth );
                widget->setColumnWidthMode( index, mode );
                widget->setColumnAlignment( index, columnAlignment );
		
                itsNumHeaders++;
	    }
        }
    } 
    else if( key == "TreeStepSize" )
    {
        widget->setTreeStepSize( value.toInt() );
    }
    else if( key == "MultiSelection" )
    {
        widget->setMultiSelection( (value == "TRUE") );
    }
    else if( key == "AllColumnsShowFocus" )
    {
        widget->setAllColumnsShowFocus( (value == "TRUE") );
    }
    else if( key == "ItemMargin" )
    {
        widget->setItemMargin( value.toInt() );
    }
    else if( key == "RootIsDecorated" )
    {
        widget->setRootIsDecorated( (value == "TRUE") );
    }
    else
    {
        retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}

void DlgListView::RestoreXML( const XMLValue &value )
{
    // Sanity check
    if( value.GetTypeString() != "ListView" )
        return;

    const XMLValueMap map = value.AsNested();

    // Create if necessary
    if( ! GetListView() )
    {
        CreateQtWidget( 0 );
    }
  
    QListView* widget = GetListView();

    XMLValueList list = map["HeaderInformation"].AsList();
    QListIterator<XMLValue> it( list );
    for( ;
         it.current();
         ++it )
    {
        const XMLValueMap header = it.current()->AsNested();

        QListView::WidthMode mode = 
            StrToWidthMode( header["WidthMode"].AsQString() );
  
        int columnWidth = (mode == QListView::Maximum) ?
            -1 :
            header["Width"].AsInt();

        if( !isFirstHeaderLoaded && (itsNumHeaders == 1) )
        {
            widget->setColumnText( 0, header["Text"].AsQString() );
            widget->setColumnWidth( 0, columnWidth );
            widget->setColumnWidthMode( 0, mode );
            widget->setColumnAlignment( 0, header["Alignment"].AsInt() );

            isFirstHeaderLoaded = TRUE;
        }
        else
        {
            int index = widget->addColumn( header["Text"].AsQString(),
                                           columnWidth );
            widget->setColumnWidthMode( index, mode );
            widget->setColumnAlignment( index, header["Alignment"].AsInt() );
		
            itsNumHeaders++;
        }
    }

    widget->setTreeStepSize( map["TreeStepSize"].AsInt() );
    widget->setMultiSelection( map["MultiSelection"].AsBool() );
    widget->setAllColumnsShowFocus( map["AllColumnsShowFocus"].AsBool() );
    widget->setItemMargin( map["ItemMargin"].AsInt() );
    widget->setRootIsDecorated( map["RootIsDecorated"].AsBool() );

    // We inherit from 
    Inherited::RestoreXML( map["ScrollView"] );

}


void DlgListView::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QListView* widget = GetListView();

    stream << varName <<
	"setTreeStepSize( " << widget->treeStepSize() << " );" << endl;
    
    stream << varName << "setMultiSelection( " <<
	(widget->isMultiSelection() ? "TRUE" : "FALSE") << " );" << endl;
    
    stream << varName << "setAllColumnsShowFocus( " <<
	(widget->allColumnsShowFocus() ? "TRUE" : "FALSE") << " );" << endl;
    
    stream << varName << 
	"setItemMargin( " << widget->itemMargin() << " );" << endl;
    
    stream << varName << "setRootIsDecorated( " << 
	(widget->rootIsDecorated() ? "TRUE" : "FALSE") << " );" << endl;
    
    unsigned int i;
    for( i = 0; i < itsNumHeaders; ++i )
    {
	stream << varName << 
	    "addColumn( ";
        GenerateText( widget->columnText( i ), widgetLayout, stream );
        stream << ", " <<
	    ((widget->columnWidthMode( i ) == QListView::Maximum) ? -1 : widget->columnWidth( i )) <<
	    " );" << endl;
	
	stream << varName << "setColumnWidthMode( " << i << ", QListView::" <<
	    WidthModeToStr( widget->columnWidthMode( i ) ) <<
	    " );" << endl;
	
	stream << varName << "setColumnAlignment( " << i << ", " <<
	    widget->columnAlignment( i ) << " );" << endl;
    }
}


void DlgListView::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgListView::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "selectionChanged()" );
	itsSignals.append( "selectionChanged(QListViewItem*)" );
	itsSignals.append( "currentChanged(QListViewItem*)" );
	itsSignals.append( "doubleClicked(QListViewItem*)" );
	itsSignals.append( "returnPressed(QListViewItem*)" );
	itsSignals.append( "rightButtonClicked(QListViewItem*,const QPoint&,int)" );
	itsSignals.append( "rightButtonPressed(QListViewItem*,const QPoint&,int)" );
    }

    return itsSignals;
}


QString DlgListView::WidthModeToStr( QListView::WidthMode theMode )
{
    QString str;
    
    switch( theMode )
    {
	case QListView::Manual:
	{
	    str = "Manual";
	    break;
	}
	case QListView::Maximum:
	{
	    str = "Maximum";
	    break;
	}
	default:
	{
	    break;
	}
    }
    
    return str;
}


QListView::WidthMode DlgListView::StrToWidthMode( 
    const QString& theWidthMode 
    )
{
    QListView::WidthMode mode = QListView::Maximum;
    
    if( theWidthMode == "Manual" )
    {
	mode = QListView::Manual;
    }
    else if( theWidthMode == "Maximum" )
    {
	mode = QListView::Maximum;
    }
    
    return mode;
}


void DlgListView::CreateQtWidget(
    const char*	theName
    )
{
    QListView* widget = 
	new QListView( GetParent(),
		       CreateName( "ListView", 
				   theName, 
				   itsNextWidgetNum ) );

    widget->addColumn( "Sample items" );
    itsNumHeaders = 1;

    QListViewItem* root1 = new QListViewItem( widget, tr( "Root item 1" ) );
    QListViewItem* child1 = new QListViewItem( root1, tr( "Child 1" ) );
    (void) new QListViewItem( child1, tr( "Subchild 1" ) );
    (void) new QListViewItem( child1, tr( "Subchild 2" ) );
    QListViewItem* child2 = new QListViewItem( root1, tr( "Child 2" ) );
    (void) new QListViewItem( child2, tr( "Subchild 1" ) );
    (void) new QListViewItem( child2, tr( "Subchild 2" ) );
    QListViewItem* root2 = new QListViewItem( widget, tr( "Root item 2" ) );
    (void) new QListViewItem( root2, tr( "Child 1" ) );
    (void) new QListViewItem( root2, tr( "Child 2" ) );

    root1->setOpen( TRUE );
    child1->setOpen( TRUE );
    child2->setOpen( TRUE );
    root2->setOpen( TRUE );

    SetQtWidget( widget );
}
