/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgScrollBar.cpp,v 2.7 2000/07/23 20:38:01 jeffharris Exp $

*******************************************************************************/


#include "widget/DlgScrollBar.h"

#include "prop/ScrollBarPage.h"

#include "GuiTools.h"

#include "xml/QtArchExpat.h"
#include "xml/QtArchGenerator.h"

#define Inherited DlgWidget

QStrList DlgScrollBar::itsSignals;
int DlgScrollBar::itsNextWidgetNum = 1;

DlgScrollBar::DlgScrollBar() :
    Inherited( DlgWidget::DW_ScrollBar )
{
}


DlgScrollBar::DlgScrollBar(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgScrollBar::~DlgScrollBar()
{
}


void DlgScrollBar::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new ScrollBarPage( this, parent, "ScrollBar" ) );

    labels.append( tr( "Scroll Bar" ) );
}

void DlgScrollBar::Save( QtArchGenerator &generator )
{
  generator.WriteStartBlock( "ScrollBar" );

  QScrollBar* widget = GetScrollBar();

  QString orientation;
  if( widget->orientation() == QScrollBar::Horizontal )
  {
    orientation = "Horizontal";
  }
  else
  { 
    orientation = "Vertical";
  }
  generator.WriteSingleLine( "Orientation", orientation );

  generator.WriteXMLValue( "MinValue", widget->minValue() );
  generator.WriteXMLValue( "MaxValue", widget->maxValue() );
  generator.WriteXMLValue( "Initial", widget->value() );
  generator.WriteXMLValue( "LineStep", widget->lineStep() );
  generator.WriteXMLValue( "PageStep", widget->pageStep() );
  generator.WriteXMLValue( "Tracking", widget->tracking() );

  Inherited::Save( generator );

  generator.WriteEndBlock();
}


bool DlgScrollBar::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    QScrollBar* widget = GetScrollBar();

    // Compare to known keys
    if( key == "Orientation" )
    {
	widget->setOrientation( 
	    ( (value == "Horizontal") ? QScrollBar::Horizontal : QScrollBar::Vertical ) );
	
    }
    else if( key == "MinValue" )
    {
	widget->setRange( value.toInt(), widget->maxValue() );
    }
    else if( key == "MaxValue" )
    {
	widget->setRange( widget->minValue(), value.toInt() );
    }
    else if( key == "Initial" )
    {
	widget->setValue( value.toInt() );
    }
    else if( key == "LineStep" )
    {
	widget->setSteps( value.toInt(), widget->pageStep() );
    }
    else if( key == "PageStep" )
    {
	widget->setSteps( widget->lineStep(), value.toInt() );
    }
    else if( key == "Tracking" )
    {
	widget->setTracking( (value == "TRUE") );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}

void DlgScrollBar::RestoreXML( const XMLValue &value )
{
  // Sanity check
  if( value.GetTypeString() != "ScrollBar" )
    return;

  const XMLValueMap map = value.AsNested();

  // Create if necessary
  if( ! GetScrollBar() )
  {
    CreateQtWidget( 0 );
  }
  
  QScrollBar* widget = GetScrollBar();

  QScrollBar::Orientation orientation;
  if( map["Orientation"].AsQString() == "Horizontal" )
    orientation = QScrollBar::Horizontal;
  else
    orientation = QScrollBar::Vertical;
  widget->setOrientation( orientation );

  widget->setRange( map["MinValue"].AsInt(), map["MaxValue"].AsInt() );
	widget->setValue( map["Initial"].AsInt() );
  widget->setSteps( map["LineStep"].AsInt(), map["PageStep"].AsInt() );


  // We inherit from 
	Inherited::RestoreXML( map["DlgWidget"] );

}


void DlgScrollBar::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QScrollBar* widget = GetScrollBar();

    stream << varName << "setOrientation( QScrollBar::" <<
	( (widget->orientation() == QScrollBar::Horizontal) ? "Horizontal" : "Vertical" ) << 
	" );" << endl;
    
    stream << varName << "setRange( " <<
	widget->minValue() << ", " << widget->maxValue() << " );" << endl;
    
    stream << varName << "setSteps( " <<
	widget->lineStep() << ", " << widget->pageStep() << " );" << endl;
    
    stream << varName << 
	"setValue( " << widget->value() << " );" << endl;

    stream << varName << "setTracking( " <<
	(widget->tracking() ? "TRUE" : "FALSE") << " );" << endl;
}


void DlgScrollBar::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgScrollBar::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "valueChanged(int)" );
	itsSignals.append( "sliderPressed()" );
	itsSignals.append( "sliderMoved(int)" );
	itsSignals.append( "sliderReleased()" );
	itsSignals.append( "nextLine()" );
	itsSignals.append( "prevLine()" );
	itsSignals.append( "nextPage()" );
	itsSignals.append( "prevPage()" );
    }

    return itsSignals;
}


void DlgScrollBar::CreateQtWidget(
    const char*	theName
    )
{
    QScrollBar* widget = 
	new QScrollBar( QScrollBar::Horizontal,
			GetParent(),
			CreateName( "ScrollBar", 
				    theName, 
				    itsNextWidgetNum ) );

    SetQtWidget( widget );
}
