/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: DlgSpinBox.cpp,v 2.10 2000/07/23 20:38:01 jeffharris Exp $

*******************************************************************************/


#include "widget/DlgSpinBox.h"

#include "prop/SpinBoxPage.h"

#include "GuiTools.h"

#include "xml/QtArchExpat.h"
#include "xml/QtArchGenerator.h"

#define Inherited DlgFrame

QStrList DlgSpinBox::itsSignals;
int DlgSpinBox::itsNextWidgetNum = 1;

DlgSpinBox::DlgSpinBox() :
    Inherited( DlgWidget::DW_SpinBox )
{
}


DlgSpinBox::DlgSpinBox(
    DlgWidget::Type	theType
    ) :
    Inherited( theType )
{
}


DlgSpinBox::~DlgSpinBox()
{
}


void DlgSpinBox::GetPropertyPages(
    QWidget*		parent,
    QList<QWidget>&	pages,
    QStrList&		labels
    )
{
    Inherited::GetPropertyPages( parent, pages, labels );

    pages.append( new SpinBoxPage( this, parent, "SpinBox" ) );

    labels.append( tr( "Spin Box" ) );
}

void DlgSpinBox::Save( QtArchGenerator &generator )
{
  generator.WriteStartBlock( "SpinBox" );

  QSpinBox* widget = GetSpinBox();

  generator.WriteXMLValue( "MinValue", widget->minValue() );
  generator.WriteXMLValue( "MaxValue", widget->maxValue() );
  generator.WriteXMLValue( "Step", widget->lineStep() );
  generator.WriteSingleLine( "Prefix", widget->prefix() );
  generator.WriteSingleLine( "Suffix", widget->suffix() );
  generator.WriteSingleLine( "SpecialValue", widget->specialValueText() );
  generator.WriteXMLValue( "Wrapping", widget->wrapping() );
  generator.WriteXMLValue( "ButtonSymbols",
                           ButtonSymbolsToStr( widget->buttonSymbols() ) );

  Inherited::Save( generator );

  generator.WriteEndBlock();
}


bool DlgSpinBox::RestoreKeyValue( 
    const QString& 	key, 
    const QString& 	value,
    bool		topLevel
    )
{
    bool retVal = TRUE;

    if( topLevel )
    {
	if( key == "Name" )
	{
	    // Update the widget number so that no duplicate names are created
	    UpdateNextWidgetNumber( value, itsNextWidgetNum );
	}
    }

    QSpinBox* widget = GetSpinBox();

    // Compare to known keys
    if( key == "MinValue" )
    {
	widget->setRange( value.toInt(), widget->maxValue() );
    } 
    else if( key == "MaxValue" )
    {
	widget->setRange( widget->minValue(), value.toInt() );
    }
    else if( key == "Step" )
    {
	widget->setSteps( value.toInt(), widget->pageStep() );
    }
    else if( key == "Prefix" )
    {
	widget->setPrefix( value );
    }
    else if( key == "Suffix" )
    {
	widget->setSuffix( value );
    }
    else if( key == "SpecialValue" )
    {
	widget->setSpecialValueText( value );
    }
    else if( key == "Wrapping" )
    {
	widget->setWrapping( (value == "TRUE") );
    }
    else
    {
	retVal = Inherited::RestoreKeyValue( key, value, FALSE );
    }

    return retVal;
}

void DlgSpinBox::RestoreXML( const XMLValue &value )
{
    // Sanity check
    if( value.GetTypeString() != "SpinBox" )
        return;

    const XMLValueMap map = value.AsNested();

    // Create if necessary
    if( ! GetSpinBox() )
    {
        CreateQtWidget( 0 );
    }
  
    QSpinBox* widget = GetSpinBox();

    widget->setRange( map["MinValue"].AsInt(), map["MaxValue"].AsInt() );
    widget->setSteps( map["Step"].AsInt(), widget->pageStep() );
    widget->setPrefix( map["Prefix"].AsQString() );
    widget->setSuffix( map["Suffix"].AsQString() );
    widget->setSpecialValueText( map["SpecialValue"].AsQString() );
    widget->setWrapping( map["Wrapping"].AsBool() );

    widget->setButtonSymbols( 
        StrToButtonSymbols( map["ButtonSymbols"].AsQString() ) );

    // We inherit from 
    Inherited::RestoreXML( map["Frame"] );

}


void DlgSpinBox::GenerateSource(
    QTextStream&		stream,
    const QString&		varName,
    const WidgetLayout&         widgetLayout
    )
{
    Inherited::GenerateSource( stream, varName, widgetLayout );

    const QSpinBox* widget = GetSpinBox();

    stream << varName << "setRange( " 
	   << widget->minValue() << ", "
	   << widget->maxValue() << " );" << endl;
    
    stream << varName << "setSteps( "
	   << widget->lineStep() << ", "
	   << "0" << " );" << endl;
    
    stream << varName << "setPrefix( ";
    GenerateText(  widget->prefix(), widgetLayout, stream );
    stream << " );" << endl;

    stream << varName << "setSuffix( ";
    GenerateText(  widget->suffix(), widgetLayout, stream );
    stream << " );" << endl;
    
    stream << varName << "setSpecialValueText( ";
    GenerateText( widget->specialValueText(), widgetLayout, stream );
    stream << " );" << endl;
    
    stream << varName << "setWrapping( "
	   << (widget->wrapping() ? "TRUE" : "FALSE") << " );" << endl;

    stream << varName << "setButtonSymbols( QSpinBox::"
           << ButtonSymbolsToStr( widget->buttonSymbols() ) << " );" << endl;
}


void DlgSpinBox::AddSignalsToList( 
    QStrList&	signalList 
    ) const
{
    Inherited::AddSignalsToList( signalList );
    GuiTools::QStrListAppendAll( signalList, GetSignalList() );
}


const QStrList& DlgSpinBox::GetSignalList()
{
    if( itsSignals.isEmpty() )
    {
	itsSignals.append( "valueChanged(int)" );
	itsSignals.append( "valueChanged(const QString&)" );
    }

    return itsSignals;
}


QString DlgSpinBox::ButtonSymbolsToStr( QSpinBox::ButtonSymbols symbols )
{
    QString str;
    
    switch( symbols )
    {
        case QSpinBox::UpDownArrows:
        {
            str = "UpDownArrows";
            break;
        }
        case QSpinBox::PlusMinus:
        {
            str = "PlusMinus";
            break;
        }
    }
    
    return str;
}


QSpinBox::ButtonSymbols DlgSpinBox::StrToButtonSymbols( const QString& str )
{
    QSpinBox::ButtonSymbols symbols = QSpinBox::UpDownArrows;
    
    if( str == "UpDownArrows" )
    {
        symbols = QSpinBox::UpDownArrows;
    }
    else if( str == "PlusMinus" )
    {
        symbols = QSpinBox::PlusMinus;
    }
    
    return symbols;
}


void DlgSpinBox::CreateQtWidget(
    const char*	theName
    )
{
    QSpinBox* widget = 
	new QSpinBox( GetParent(),
		      CreateName( "SpinBox", 
				  theName, 
				  itsNextWidgetNum ) );

    SetQtWidget( widget );
}
