/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: QtArchExpat.cpp,v 1.11 2000/07/23 20:38:02 jeffharris Exp $

*******************************************************************************/

#include "QtArchExpat.h"
#include <qfile.h>
#include <qtextstream.h>
#include <qcstring.h>



/*
  QtArchExpat expects a XML file with the following sub-structures
  simple type
  complex type
     simple type
     complex type
  nested
     simple type
     complex type
     nested type
     dlgwidget type
  QtArch
     simple type
     nested type
     complex type
*/     

#define BLOCKSIZE 1024

extern "C"
{
  static QtArchExpat *qtArchExpat = 0;

  void StartElement( void *, const XML_Char *, const XML_Char ** );
  void EndElement( void *, const XML_Char * );
  void CharacterData( void *, const XML_Char *, int );
  void ProcessingInstruction( void *, const XML_Char *, const XML_Char * );
  void Comment( void *, const XML_Char * );
};

QtArchExpat::QtArchExpat( const QString &filename )
{
  theFileName = filename;

  // Setting this as the object to call
  qtArchExpat = this;

  // Creating the parser w/o encoding informations
  theParser = XML_ParserCreate( 0 );

  // Set up callbacks
  XML_SetElementHandler( theParser, ::StartElement, ::EndElement );
  XML_SetCharacterDataHandler( theParser, ::CharacterData );
  XML_SetProcessingInstructionHandler( theParser, ::ProcessingInstruction );
  XML_SetCommentHandler( theParser, ::Comment );

  // Set nested stack
  nested.setAutoDelete( TRUE );
  list.setAutoDelete( TRUE );
  types.setAutoDelete( TRUE );
  complex.setAutoDelete( TRUE );
  widgets.setAutoDelete( TRUE );
  superType.setAutoDelete( TRUE );
  
  simpleMap.setAutoDelete( TRUE );
  simpleList.setAutoDelete( TRUE );

  // No layout
  layoutFlag = FALSE;
}

QtArchExpat::~QtArchExpat()
{
  // Free up the parser
  XML_ParserFree( theParser );
}

bool QtArchExpat::Parse()
{
  QFile file( theFileName );
  
  file.open( IO_ReadOnly );

  // We didn't even have started
  parsedEnough = FALSE;
  
  while( ! (file.atEnd() || parsedEnough ) )
  {
    char buffer[BLOCKSIZE+1];
    int len = file.readBlock( buffer, BLOCKSIZE );

    if( !XML_Parse( theParser, buffer, len, len < BLOCKSIZE ) ) 
    {
      fprintf(stderr,
              "%s at line %d\n",
              XML_ErrorString( XML_GetErrorCode( theParser ) ),
              XML_GetCurrentLineNumber( theParser ) );
      return FALSE;
    }
  }

  return TRUE;
}

QString QtArchExpat::GetType()
{
  connect( this,
           SIGNAL(NewDlgObject(const StringMap&)),
           this,
           SLOT(GetTypeSlot(const StringMap&)) );
  
  // Normal parse
  Parse();

  // This was set in the slot GetTypeSlot
  return qtArchType;
}

void QtArchExpat::GetTypeSlot( const StringMap &attrMap )
{
  // Parser might stop any time now
  parsedEnough = TRUE;

  qtArchType = attrMap["type"];
}

void QtArchExpat::StartElement( UData /*udata*/,
                                const XML_Char *name,
                                const XML_Char **atts )
{
  bool *nestedFlag = new bool;
  bool *listFlag = new bool;
  bool complexFlag, dlgwidgetFlag;

  *nestedFlag = FALSE;
  *listFlag = FALSE;
  complexFlag = FALSE;
  dlgwidgetFlag = FALSE;

  // Getting all the attributes
  StringMap attrMap;
  while( *atts )
  {
#ifdef DEBUGEXPAT
    debug( "Atts %s = %s", (const char *) *atts, (const char *) *(atts+1) );
#endif
    attrMap[*atts] = *(atts+1);
    atts += 2;
  }

  // Scanning for special attributes
  if( attrMap.contains( "nested" ) && attrMap["nested"] == "true" )
    *nestedFlag = TRUE;
  if( attrMap.contains( "list" ) && attrMap["list"] == "true" )
    *listFlag = TRUE;
  if( attrMap.contains( "complex" ) && attrMap["complex"] == "true" )
    complexFlag = TRUE;
  if( attrMap.contains( "supertype" ) && attrMap["supertype"] == "true" )
    superType.push( new QString( name ) );
  if( attrMap.contains( "type" ) )
    currentType = attrMap["type"];
  else
    currentType = "";

  // Record the nested an list flag foreach element in the tree
  nested.push( nestedFlag );
  list.push( listFlag );

  // When starting a complex type (Any type known to XMLValue), record
  // the type and the name
  if( complexFlag )
  {
    types.push( new QString( currentType ) );
    complex.push( new QString( name ) );
  }

  // If this type is nested, we need a map of simpler types to store its
  // values
  if( *nestedFlag && ! *listFlag )
  {
    simpleMap.push( new XMLValueMap );
  }
  
  // If this type is a list, we need a list of simpler types to store its
  // values
  if( *listFlag )
  {
    simpleList.push( new XMLValueList );
  }
  
  currentElement = name;

  // Reset currentData
  currentData = "";
  
  // Check for dlg objects
  if( currentElement == "QtArch" )
  {
    emit NewDlgObject( attrMap );
  }
    
}

void QtArchExpat::EndElement( UData /*udata*/, const XML_Char *name )
{
    XMLValue value;
    value.SetName( name );

#ifdef DEBUGEXPAT
    debug( "Ending element >%s<", name );
#endif

    // Ending a formal super type?
    if( superType.count() && (*(superType.top()) == QString( name )) )
        superType.remove();
    // Only top level super types are of interest
    bool isSuperType = superType.count() ? TRUE : FALSE;

    // Ending a list?
    bool useList = *list.top();
    list.remove();
    // Was the parent a list?
    bool parentUseList = list.isEmpty() ? FALSE : *list.top();

#ifdef DEBUGEXPAT
    debug( "Finishing %s, useList = %s, parentUseList = %s", name,
           ( useList ? "Yes" : "No" ), ( parentUseList ? "Yes" : "No" ) );
#endif

    // Finishing a complex type
    if( ! complex.isEmpty() && *(complex.top()) == name )
    {
        // A complex type is always child in a nested parent, so we just store
        // the map in the parent
        if( useList )
        {
            value.SetTypValue( *(types.top()), *(simpleList.top()) );
            simpleList.remove();
        }
        else
        {
            value.SetTypValue( *(types.top()), *(simpleMap.top()) ); 
            simpleMap.remove();
        }
        
        if( parentUseList )
            (*simpleList.top()).append( new XMLValue( value ));
        else
            (*simpleMap.top())[name] = value;
    
        // Remove the toplevel complex 
        complex.remove();
        types.remove();
    } else if( *(nested.top() ) )
    {
        // Finishing a nested, not complex type or a layout
        if( ! isSuperType )      
        {
            if( useList )
            {
                value.SetTypValue( name, *(simpleList.top()) );
                simpleList.remove();
            }
            else
            {
                value.SetTypValue( name, *(simpleMap.top()) );
                simpleMap.remove();
            }

#ifdef DEBUGEXPAT
            debug( "Emiting nested, not complex %s (%s list)",
                   name, useList ? "" : "not a " );
#endif
            emit NewType( value );
        } else
        {
#ifdef DEBUGEXPAT
            debug( "Not complex, layout = %s", layoutFlag ? "Yes" : "No" );
#endif
            if( useList )
            {
                value.SetTypValue( name, *(simpleList.top()) );
                simpleList.remove();
            }
            else
            {
                value.SetTypValue( name, *(simpleMap.top()) );
                simpleMap.remove();
            }
            
            if( parentUseList )
                (*simpleList.top()).append( new XMLValue( value ) );
            else
                (*simpleMap.top())[name] = value;
        }
    } else
    {
        // A simple value in a complex or nested environment
        value.SetTypValue( currentType, currentData );

        if( parentUseList )
            (*simpleList.top()).append( new XMLValue( value ) );
        else
            (*simpleMap.top())[currentElement] = value;
    }

    // Reset currentData
    currentData = "";
  
    nested.remove();
  
}

void QtArchExpat::CharacterData( UData /*udata*/, const XML_Char *ptr, int len )
{
    currentData += QString::fromUtf8( ptr, len );
}

void QtArchExpat::ProcessingInstruction( UData /*udata*/, 
                                         const XML_Char */*target*/, 
                                         const XML_Char */*data*/ )
{
}

void QtArchExpat::Comment( UData /*udata*/, const XML_Char */*data*/ )
{
}

void StartElement( void *uData, const XML_Char *name, const XML_Char **atts )
{
  if( qtArchExpat )
    qtArchExpat->StartElement( uData, name, atts );
}

void EndElement( void *uData, const XML_Char *name )
{
  if( qtArchExpat )
    qtArchExpat->EndElement( uData, name );
}

void CharacterData( void *uData, const XML_Char *ptr, int len )
{
  if( qtArchExpat )
    qtArchExpat->CharacterData( uData, ptr, len );
}

void ProcessingInstruction( void *uData, const XML_Char *target, const XML_Char *data )
{
  if( qtArchExpat )
    qtArchExpat->ProcessingInstruction( uData, target, data );
}

void Comment( void *uData, const XML_Char *data )
{
  if( qtArchExpat )
    qtArchExpat->Comment( uData, data );
}





