/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: QtArchGenerator.cpp,v 1.7 2000/07/23 20:38:02 jeffharris Exp $

*******************************************************************************/

#include "QtArchGenerator.h"
#include <qregexp.h>
#include <qtextcodec.h>


QtArchGenerator::QtArchGenerator( bool pD )
{
  indentionLevel = 0;
  stream = NULL;
  ourStream = FALSE;
  printDefaults = pD;
}


QtArchGenerator::~QtArchGenerator()
{
  if( ourStream )
    delete stream;
}


void QtArchGenerator::WriteStart( const QString &fileName )
{
  file.setName( fileName );
  if( ! file.open( IO_WriteOnly ) )
    {
      GeneratorError err( "Couldn't open file for writing" );
      throw err;
    }

  stream = new QTextStream( &file );
  ourStream = TRUE;
  
  *stream << "<?xml version=\"1.0\" encoding=\"UTF-8\"?>" << endl;

}

void QtArchGenerator::WriteStart( QTextStream *str )
{
  stream = str;
  
  *str << "<?xml version='1.0'?>" << endl;
}


void QtArchGenerator::WriteStartBlock( const QString &name,
                                       bool nested,
                                       const QString &atts )
{
  elementStack.push( new QString( name ) );
  
  QString a( atts );
  if( nested )
    a += " nested=\"true\" ";
  
  *stream << indent() << "<" << name << " " << a << ">" << endl;

  indentionLevel++;

  if( file.isOpen() && file.status() )
    {
      GeneratorError err( "Error while writing to file" );
      throw err;
    }

}

void QtArchGenerator::WriteStartList( const QString &name,
                                      const QString &atts )
{
  QString a( atts );
  a += " list=\"true\" ";

  WriteStartBlock( name, TRUE, a );
}


void QtArchGenerator::WriteEndBlock()
{
  QString *name = elementStack.pop();

  indentionLevel--;

  *stream << indent() << "</" << *name << ">" << endl;

  delete name;

  if( file.isOpen() && file.status() )
    {
      GeneratorError err( "Error while writing to file" );
      throw err;
    }

}


void QtArchGenerator::WriteSingleLine( const QString &name, const QString &value, const QString &atts )
{
  if( ! printDefaults && value.isEmpty() )
    return;
  
  *stream << indent() << "<" << name << " " << atts << ">" << 
    EncodeText( value ) << "</" << name << ">" << endl;

  if( file.isOpen() && file.status() )
    {
      GeneratorError err( "Error while writing to file" );
      throw err;
    }

}


void QtArchGenerator::WriteEnd()
{
  if( elementStack.count() > 0 )
    {
      GeneratorError err( "Unbalanced xml file generated" );

      throw err;
    }

  if( file.isOpen() )
    file.close();
}

void QtArchGenerator::WriteXMLValue( const QString &name,
                                     const XMLValue &value )
{
  if( ! printDefaults && value.IsDefault() )
    return;
  
  QString attr = "type=\"";

  attr += value.GetTypeString();
  attr += "\"";

  // Complex type will get broken down
  if( value.ComplexType() )
  {
    attr += " complex=\"true\" ";
    WriteStartBlock( name, TRUE, attr );
    BreakDownXMLValue( value );
    WriteEndBlock();
  } else
  {
    WriteSingleLine( name, value.AsQString(), attr );
  }
}

void QtArchGenerator::BreakDownXMLValue( const XMLValue &value )
{
  XMLValueMap map = value.BreakDown();
  XMLValueMap::Iterator val;

  // The following loop could become recursive
  for( val = map.begin(); val != map.end(); ++val )
  {
    WriteXMLValue( val.key(), val.data() );
  }
}

QString QtArchGenerator::indent( )
{
  QString retStr;

  retStr.fill( ' ', indentionLevel * 2 );

  return retStr;
}

QString QtArchGenerator::EncodeText( const QString& text )
{
    QString str;

    unsigned int i;
    for( i = 0; i < text.length(); ++i )
    {
        unsigned short unicode = text[i].unicode();
        if( unicode > 0x80 )
        {
            str += "&#x";
            str += QString::number( unicode, 16 );
            str += ";";
        }
        else
        {
            switch( unicode & 0xff )
            {
                // Do & first because other tests will add &'s
                case '&':
                {
                    str += "&amp;";
                    break;
                }
                case '<':
                {
                    str += "&lt;";
                    break;
                }
                case '>':
                {
                    str += "&gt;";
                    break;
                }
                case '\'':
                {
                    str += "&apos;";
                    break;
                }
                case '\"':
                {
                    str += "&quot;";
                    break;
                }
                default:
                {
                    str.append( (char)(unicode & 0xff) );
                    break;
                }
            }
        }
    }

    return str;
}
