/*******************************************************************************

    Qt Architect

    Copyright (C) 1997-2000 Jeff Harris and Klaus Ebner

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

    $Id: XMLValue.h,v 1.7 2000/07/23 20:38:02 jeffharris Exp $

*******************************************************************************/

#ifndef XMLValue_included
#define XMLValue_included

#include <qmap.h>
#include <qstring.h>
#include <qsize.h>
#include <qrect.h>
#include <qpalette.h>
#include <qfont.h>
#include <qcolor.h>
#include <qlist.h>

class XMLValue;
class QtArchGenerator;

class XMLValueMap : public QMap<QString,XMLValue>
{
public:

    XMLValueMap();
    // Constructor

    XMLValueMap( const XMLValueMap& map );
    // Copy constructor
    
    XMLValueMap& operator=( const XMLValueMap& map );
    // Assignment operator
};


class XMLValueList : public QList<XMLValue>
{
public:
  typedef QListIterator<XMLValue> Iterator;
    
    XMLValueList();
    // Constructor

    XMLValueList( const XMLValueList& list );
    // Copy constructor

    XMLValueList& operator=( const XMLValueList& list );
    // Assignment operator
};

class XMLValue
{
public:
  enum XMLValueType 
  {
    XMLNoType, XMLint, XMLbool, XMLdouble,XMLQString, XMLQSize, XMLQRect, 
    XMLQPoint, XMLQPalette, XMLQFont, XMLQColor, XMLQColorGroup, XMLDlgWidget, 
    XMLNested, XMLList
  };

public:
  XMLValue();
  XMLValue( const XMLValue & );
  XMLValue( const int );
  XMLValue( const bool );
  XMLValue( const double );
  XMLValue( const QString & );
  XMLValue( const QSize & );
  XMLValue( const QRect & );
  XMLValue( const QPoint & );
  XMLValue( const QPalette & );
  XMLValue( const QFont & );
  XMLValue( const QColor & );
  XMLValue( const QColorGroup & );

  // ctor using verbatim type string and a stringified value
  // This is suitable only for simple types
  XMLValue( const QString &, const QString & );
  
  // ctor using verbatim type string and a map of values
  // Intended for complex types and unknow structure types
  XMLValue( const QString &, const XMLValueMap &, XMLValueType = XMLNested );

  // ctor using verbatim type string and a list of values
  XMLValue( const QString &, const XMLValueList & );

    ~XMLValue();

    XMLValue& operator=( const XMLValue& copyValue );
  
  XMLValueType Type() const;
  void SetType( XMLValueType );
  void SetTypValue( const QString &, const QString & );
  void SetTypValue( const QString &, const XMLValueMap &,
                    XMLValueType = XMLNested );
  void SetTypValue( const QString &, const XMLValueList & );

  void SetName( const QString & );
  const QString &GetName() const;
  
  void SetInt( const int );
  void SetBool( const bool );
  void SetDouble( const double );
  void SetNested( const XMLValueMap &, XMLValueType = XMLNested );
  void SetQString( const QString & );
  void SetQSize( const QSize & );
  void SetQRect( const QRect & );
  void SetQPoint( const QPoint & );
  void SetQPalette( const QPalette & );
  void SetQFont( const QFont & );
  void SetQColor( const QColor & );
  void SetQColorGroup( const QColorGroup & );

  int AsInt() const;
  bool AsBool() const;
  double AsDouble() const;
  QString AsQString() const;
  QSize AsQSize() const;
  QRect AsQRect() const;
  QPoint AsQPoint() const;
  QPalette AsQPalette() const;
  QFont AsQFont() const;
  QColor AsQColor() const;
  QColorGroup AsQColorGroup() const;
  XMLValueMap AsNested() const;
  XMLValueList AsList() const;

  // Check if the XMLValue simply represents the XMLTypes default
  bool IsDefault() const;
  
  // Identify and simplify complex data types
  bool ComplexType() const;
  XMLValueMap BreakDown() const;
  
  QString GetTypeString( XMLValueType = XMLNoType ) const;

  void Dump( const QString & );
  // For debugging: Dump the whole XML structure to the given file
  
private: // Methods

    void _Dump( QtArchGenerator * ) const;
    
    void InitValue();

private: // Attributes
  
  struct Value
  {
    int intValue;
    bool boolValue;
    double doubleValue;
    QString qstringValue;
    QSize qsizeValue;
    QRect qrectValue;
    QPoint qpointValue;
    QPalette qpaletteValue;
    QFont qfontValue;
    QColor qcolorValue;
    QColorGroup qcolorgroupValue;
    XMLValueMap map;
    XMLValueList list;
  } *value;

  XMLValueType type;
  QString typeString;
  bool complexType;
  QString name;
};

inline void XMLValue::SetName( const QString &n )
{
  name = n;
}

inline const QString &XMLValue::GetName() const
{
  return name;
}


inline void XMLValue::InitValue()
{
    delete value;
    value = new Value;
}

#endif
